//*******************************************************
//
//	MP3 Player in SEGA Visual Memory
//
//	シリアル通信
//
//	ATmega1284p
//		efuse:0xFD	BOD 2.7V
//		hfuse:0xD1	JTAG禁止, EEPROM消去保護
//		lfuse:0xE2	内蔵CR発振 8MHz
//
//	Compiler:	AVR-GCC		4.3.5
//				AVR-LIBC	1.6.8
//				binutils	2.20
//
//	2012.5.20	KAWAKAMI Yukio
//*******************************************************

#include	<string.h>
#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>

#include	"main.h"
#include	"com.h"

extern volatile COM_WORK Com0;
extern volatile COM_WORK Com1;

PROGMEM ushort BaudReg[12] = {
	(ushort)(F_CPU / 16 / 300L - 1),
	(ushort)(F_CPU / 16 / 600L - 1),
	(ushort)(F_CPU / 16 / 1200L - 1),
	(ushort)(F_CPU / 16 / 2400L - 1),
	(ushort)(F_CPU / 16 / 4800L - 1),
	(ushort)(F_CPU / 16 / 9600L - 1),
	(ushort)(F_CPU / 16 / 14400L - 1),
	(ushort)(F_CPU / 16 / 19200L - 1),
	(ushort)(F_CPU / 16 / 28800L - 1),
	(ushort)(F_CPU / 16 / 38400L - 1),
	(ushort)(F_CPU / 16 / 57600L - 1),
	(ushort)(F_CPU / 16 / 115200L - 1)
};


//********************************************************************
//	シリアル 受信割り込み (Com0)
//********************************************************************
ISR(USART0_RX_vect){

	if (UCSR0A & ((1<<FE0)|(1<<DOR0)|(1<<UPE0))){	// 受信エラー
		unsigned char	dummy;
		dummy = UDR0;	// 読み捨て
	} else {
		Com0.ibuff[Com0.ibuff_in++] = UDR0;
		if (Com0.ibuff_in >= sizeof(Com0.ibuff)){
			Com0.ibuff_in = 0;
		}
		(Com0.ibuff_remain)++;
	}
}


//********************************************************************
//	シリアル 受信割り込み (Com1)
//********************************************************************
ISR(USART1_RX_vect){

	if (UCSR1A & ((1<<FE1)|(1<<DOR1)|(1<<UPE1))){	// 受信エラー
		unsigned char	dummy;
		dummy = UDR1;	// 読み捨て
	} else {
		Com1.ibuff[Com1.ibuff_in++] = UDR1;
		if (Com1.ibuff_in >= sizeof(Com1.ibuff)){
			Com1.ibuff_in = 0;
		}
		(Com1.ibuff_remain)++;
	}
}


#if TX_BUFF_SIZE > 0
//********************************************************************
//	シリアル送信データレジスタ空き割り込み (Com0)
//********************************************************************
ISR(USART0_UDRE_vect){

	if (Com0.obuff_remain){
		UDR0 = Com0.obuff[Com0.obuff_out++];
		if (Com0.obuff_out >= sizeof(Com0.obuff)){
			Com0.obuff_out = 0;
		}
		--(Com0.obuff_remain);
	}

	if (!Com0.obuff_remain){
		UCSR0B &= ~(1<<UDRIE0);	// 送信データレジスタ空き割り込み禁止
	}
}


//********************************************************************
//	シリアル送信データレジスタ空き割り込み (Com1)
//********************************************************************
ISR(USART1_UDRE_vect){

	if (Com1.obuff_remain){
		UDR1 = Com1.obuff[Com1.obuff_out++];
		if (Com1.obuff_out >= sizeof(Com1.obuff)){
			Com1.obuff_out = 0;
		}
		--(Com1.obuff_remain);
	}

	if (!Com1.obuff_remain){
		UCSR1B &= ~(1<<UDRIE1);	// 送信データレジスタ空き割り込み禁止
	}
}
#endif


//********************************************************************
//	シリアルポート初期化
//********************************************************************
void init_com(volatile COM_WORK *com, COM_BAUD baud){

	cli();	// 割り込み禁止

	if (com == &Com0){
		ushort	b;
		b = pgm_read_word(&BaudReg[baud]);
		UBRR0H = (uchar)(b >> 8);
		UBRR0L = (uchar)b;
		UCSR0A = 0;
		UCSR0B = (1<<RXCIE0)|(1<<RXEN0)|(1<<TXEN0);	// 送受信許可、受信完了割り込み許可
		UCSR0C = (1<<UCSZ01)|(1<<UCSZ00);	// Stop1 8bit non
	} else if (com == &Com1){
		ushort	b;
		b = pgm_read_word(&BaudReg[baud]);
		UBRR1H = (uchar)(b >> 8);
		UBRR1L = (uchar)b;
		UCSR1A = 0;
		UCSR1B = (1<<RXCIE1)|(1<<RXEN1)|(1<<TXEN1);	// 送受信許可、受信完了割り込み許可
		UCSR1C = (1<<UCSZ11)|(1<<UCSZ10);	// Stop1 8bit non
	}

	memset((void *)com, 0, sizeof(COM_WORK));

	sei();	// 割り込み許可
}


//********************************************************************
//	受信バッファに入っているデータの byte数取得
//********************************************************************
uchar check_com(volatile COM_WORK *com){

	return com->ibuff_remain;
}


//********************************************************************
//	シリアルポート受信バッファから 1byte読み込み
//********************************************************************
unsigned char read_com(volatile COM_WORK *com){
	unsigned char	data;

	// 受信データが無ければ来るまで待つ
	while(!com->ibuff_remain){
		WDR;
		SLEEP;
	}

	// 受信バッファからデータ読み出し
	data = com->ibuff[com->ibuff_out++];
	if (com->ibuff_out >= sizeof(com->ibuff)){
		com->ibuff_out = 0;
	}
	cli();
	--com->ibuff_remain;
	sei();
	
	return data;
}


//********************************************************************
//	シリアルポート 1byte送信
//********************************************************************
void write_com(volatile COM_WORK *com, unsigned char data){

#if TX_BUFF_SIZE > 0
	// 送信バッファがいっぱいなら待つ
	while(com->obuff_remain >= sizeof(com->obuff)){
		WDR;
		SLEEP;
	}

	// データを送信バッファに積む
	com->obuff[com->obuff_in++] = data;
	if (com->obuff_in >= sizeof(com->obuff)){
		com->obuff_in = 0;
	}
	cli();
	com->obuff_remain++;
	sei();

	if (com == &Com0){
		// 送信データレジスタ空き割り込み許可
		UCSR0B |= (1<<UDRIE0);
	} else if (com == &Com1){
		// 送信データレジスタ空き割り込み許可
		UCSR1B |= (1<<UDRIE1);
	}

#else
	if (com == &Com0){
		// 送信データレジスタ空きフラグチェック
		while(!(UCSR0A & (1<<UDRE0)));
		UDR0 = data;
	} else if (com == &Com1){
		// 送信データレジスタ空きフラグチェック
		while(!(UCSR1A & (1<<UDRE1)));
		UDR1 = data;
	}
#endif
}


//********************************************************************
//	RAM上文字列送信
//********************************************************************
void write_str(volatile COM_WORK *com, uchar *str){
	unsigned char	d;

	while((d = *str++)){
		write_com(com, d);
	}
}


//********************************************************************
//	固定文字列送信
//********************************************************************
void write_str_P(volatile COM_WORK *com, PGM_P str){
	unsigned char	d;

	while((d = pgm_read_byte(str++))){
		write_com(com, d);
	}
}


