#ifndef __COM_H__
#define	__COM_H__
//*******************************************************
//
//	MP3 Player in SEGA Visual Memory
//
//	シリアル通信
//
//	ATmega1284p
//		efuse:0xFD	BOD 2.7V
//		hfuse:0xD1	JTAG禁止, EEPROM消去保護
//		lfuse:0xE2	内蔵CR発振 8MHz
//
//	Compiler:	AVR-GCC		4.3.5
//				AVR-LIBC	1.6.8
//				binutils	2.20
//
//	2012.5.20	KAWAKAMI Yukio
//*******************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>

#define	RX_BUFF_SIZE	32
#define	TX_BUFF_SIZE	32

typedef struct S_COM_WORK {
	volatile uchar	ibuff[RX_BUFF_SIZE];
	volatile uchar	ibuff_in;
	volatile uchar	ibuff_out;
	volatile uchar	ibuff_remain;

#if TX_BUFF_SIZE > 0
	volatile uchar	obuff[TX_BUFF_SIZE];
	volatile uchar	obuff_in;
	volatile uchar	obuff_out;
	volatile uchar	obuff_remain;
#endif
} COM_WORK;

typedef enum E_COM_BAUD {
	B300 = 0,
	B600,
	B1200,
	B2400,
	B4800,
	B9600,
	B14400,
	B19200,
	B28800,
	B38400,
	B57600,
	B115200
} COM_BAUD;


void	init_com(volatile COM_WORK *com, COM_BAUD baud);
uchar	check_com(volatile COM_WORK *com);
uchar	read_com(volatile COM_WORK *com);
void	write_com(volatile COM_WORK *com, uchar data);
void	write_str(volatile COM_WORK *com, uchar *str);
void	write_str_P(volatile COM_WORK *com, PGM_P str);

#endif	// __COM_H__
