//*******************************************************
//
//	MP3 Player in SEGA Visual Memory
//
//	グラフィック・漢字表示関係
//
//	ATmega1284p
//		efuse:0xFD	BOD 2.7V
//		hfuse:0xD1	JTAG禁止, EEPROM消去保護
//		lfuse:0xE2	内蔵CR発振 8MHz
//
//	Compiler:	AVR-GCC		4.3.5
//				AVR-LIBC	1.6.8
//				binutils	2.20
//
//	2012.6.10	KAWAKAMI Yukio
//*******************************************************
#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>
#include	<avr/eeprom.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>


#include	"main.h"
#include	"lcd.h"
#include	"graph.h"
#include	"rom.h"
#include	"ff.h"

extern int		Color, BGcolor;


//*******************************************************
//	GRPリソース表示
//*******************************************************
void put_grp(short xs, short ys, uchar res){
	GRPHEADER	grp_header;
	int		width, height;
	long	size;
	uchar	remain = 0;
	ushort	buff[32], *p;
	ushort	comp1, comp2;
	ushort	repeat = 0;

	ulong	adrs, data_size;
	
	if (resource_adrs_size(res, &adrs, &data_size))	return;

	// GRPヘッダ読み込み
	ROM_read(adrs, (uchar *)&grp_header, sizeof(grp_header));
	adrs += sizeof(grp_header);
	
	// ファイルヘッダチェック
	if (grp_header.mark != 0x5047){ goto EXIT; }	// ファイルタイプエラー

	// ファイルフォーマットチェック
	width = grp_header.width;
	height = grp_header.height;
	comp1 = grp_header.comp1;
	comp2 = grp_header.comp2;
	if ((grp_header.type != 1)||
		(width > 128)||
		(height > 128)){
		goto EXIT;
	}

	// 画面内に収まるか
	if ((xs + width > LCD_WIDTH)||
		(ys + height > LCD_HEIGHT)){
		goto EXIT;
	}

	// 液晶データウィンドウ
	set_window(xs, ys, xs + width - 1, ys + height - 1);

	size = width;
	size *= height;

	LCD_DATA;
	LCD_CS_L;

	p = buff;
	while(size > 0){
		ushort	data;;

		WDR;

		if (remain == 0){
			ROM_read(adrs, (uchar *)buff, sizeof(buff));
			remain = sizeof(buff) / sizeof(ushort);
			p = (ushort *)buff;
			adrs += sizeof(buff);
		}
		data = *p++;
		--remain;

		if (comp1 && (comp1 == data)){	// ランレングス圧縮マーク
			if (remain == 0){
				ROM_read(adrs, (uchar *)buff, sizeof(buff));
				remain = sizeof(buff) / sizeof(ushort);
				p = (ushort *)buff;
				adrs += sizeof(buff);
			}
			repeat = *p++;
			--remain;
			if (remain == 0){
				ROM_read(adrs, (uchar *)buff, sizeof(buff));
				remain = sizeof(buff) / sizeof(ushort);
				p = (ushort *)buff;
				adrs += sizeof(buff);
			}
			data = *p++;
			--remain;
			while(repeat){
				LCD_PORT = data >> 8;
				LCD_WR_L;			// WR=L
				LCD_WR_H;			// WR=H
				LCD_PORT = (uchar)data;
				LCD_WR_L;			// WR=L
				LCD_WR_H;			// WR=H
				--size;
				--repeat;
			}
		} else {
			LCD_PORT = data >> 8;
			LCD_WR_L;			// WR=L
			LCD_WR_H;			// WR=H
			LCD_PORT = (uchar)data;
			LCD_WR_L;			// WR=L
			LCD_WR_H;			// WR=H
			--size;
			--repeat;
		}
	}

  EXIT:
	LCD_CS_H;
}


//*******************************************************
//	GRPファイル表示
//*******************************************************
char load_grp(char *fname, int x, int y){
	FIL	file;
	GRPHEADER	grp_header;
	int	width, height;
	FRESULT	res;
	UINT	n;
	long	size;
	char	ret = 0;
	uchar	remain = 0;
	ushort	buff[32], *p;
	ushort	comp1, comp2;
	ushort	repeat = 0;


	// ファイルオープン
	if ((res = f_open(&file, fname, FA_READ) != FR_OK))	return res;	// ファイルオープンエラー

	// ファイルヘッダ読み込み
	f_read(&file, (void *)&grp_header, sizeof(grp_header), &n);

	// ファイルヘッダチェック
	if (grp_header.mark != 0x5047){ ret = 20; goto EXIT; }	// ファイルタイプエラー

	// ファイルフォーマットチェック
	width = grp_header.width;
	height = grp_header.height;
	comp1 = grp_header.comp1;
	comp2 = grp_header.comp2;
	if ((grp_header.type != 1)||
		(width > 128)||
		(height > 128)){
		ret = 30;
		goto EXIT;
	}

	// 画面内に収まるか
	if ((x + width > LCD_WIDTH)||
		(y + height > LCD_HEIGHT)){
		ret = 40;
		goto EXIT;
	}

	// 液晶データウィンドウ
	set_window(x, y, x + width - 1, y + height - 1);

	size = width;
	size *= height;

	LCD_DATA;
	LCD_CS_L;

	p = buff;
	while(size > 0){
		ushort	data;;

		WDR;

		if (remain == 0){
			f_read(&file, (void *)buff, sizeof(buff), &n);
			if (n <= 0){ ret = 50; goto EXIT; }	// 読み込みエラー
			remain = n / sizeof(ushort);
			p = buff;
		}
		data = *p++;
		--remain;

		if (comp1 && (comp1 == data)){	// ランレングス圧縮マーク
			if (remain == 0){
				f_read(&file, (void *)buff, sizeof(buff), &n);
				if (n <= 0){ ret = 50; goto EXIT; }	// 読み込みエラー
				remain = n / sizeof(ushort);
				p = buff;
			}
			repeat = *p++;
			--remain;
			if (remain == 0){
				f_read(&file, (void *)buff, sizeof(buff), &n);
				if (n <= 0){ ret = 50; goto EXIT; }	// 読み込みエラー
				remain = n / sizeof(ushort);
				p = buff;
			}
			data = *p++;
			--remain;
			while(repeat){
				LCD_PORT = data >> 8;
				LCD_WR_L;			// WR=L
				LCD_WR_H;			// WR=H
				LCD_PORT = (uchar)data;
				LCD_WR_L;			// WR=L
				LCD_WR_H;			// WR=H
				--size;
				--repeat;
			}
		} else {
			LCD_PORT = data >> 8;
			LCD_WR_L;			// WR=L
			LCD_WR_H;			// WR=H
			LCD_PORT = (uchar)data;
			LCD_WR_L;			// WR=L
			LCD_WR_H;			// WR=H
			--size;
			--repeat;
		}
	}

  EXIT:
	LCD_CS_H;
	f_close(&file);

	return ret;
}


//*******************************************************
//	UTF-8文字列表示
//*******************************************************
void print_utf8(short xs, short y, short width, uchar font, char *str){
	ushort		code;
	uchar		c;
	uchar		remain;
	short		x;
	short		right;

	if (width < 0)	width = LCD_WIDTH;

	right = xs + width;
	if (right > LCD_WIDTH)	right = LCD_WIDTH;
	
	x = xs;
	remain = 0;
	code = 0;
	while((c = *str++)){
		if (c == 0){
			break;
		} else if (c == LF){	// 改行
			x = xs;
			y += font + LINE_SPACE;
		} else if (c == CR){
			x = xs;
		} else if (c < 0x80){
			if ((x + font / 2) > right){
				x = xs;
				y += font + LINE_SPACE;
				if (y >= LCD_HEIGHT)	break;
			}
			putkj(x, y, font, c);
			x += font / 2;
		} else if ((c & 0xE0) == 0xC0){
			code = c & 0x1F;
			remain = 1;
		} else if ((c & 0xF0) == 0xE0){
			code = c & 0x0F;
			remain = 2;
		} else if ((c & 0xC0) == 0x80){
			code <<= 6;
			code |= (c & 0x3F);
			if (--remain == 0){
				uchar	w;
				if (code == 0xFEFF)	continue;	// BOM
				code = unicode_to_sjis(code);
				if (code > 0x00FF){
					w = font;
				} else {
					w = font / 2;
				}
				putkj(x, y, font, code);
				if ((x + w) > right){
					x = xs;
					y += font + LINE_SPACE;
					if (y >= LCD_HEIGHT)	break;
				}
				x += w;
			}
		}
	}
}


//*******************************************************
//	UTF-8文字列の表示に必要な領域サイズ取得
//*******************************************************
short check_utf8_area_size(char *str, uchar font, short max_width, short *width_p, short *height_p){
	ushort		code;
	uchar		c;
	uchar		remain;
	short		x, y, width;

	x = y = width = 0;
	remain = 0;
	code = 0;
	while((c = (uchar)(*str++))){
		if (c == 0){
			break;
		} else if (c == LF){	// 改行
			if (x > width)	width = x;
			x = 0;
			y += font + LINE_SPACE;
		} else if (c == CR){
			if (x > width)	width = x;
			x = 0;
		} else if (c < 0x80){
			if ((x + font / 2) > max_width){
				if (x > width)	width = x;
				x = 0;
				y += font + LINE_SPACE;
				if (y >= LCD_HEIGHT)	break;
			}
			x += font / 2;
		} else if ((c & 0xE0) == 0xC0){
			code = c & 0x1F;
			remain = 1;
		} else if ((c & 0xF0) == 0xE0){
			code = c & 0x0F;
			remain = 2;
		} else if ((c & 0xC0) == 0x80){
			code <<= 6;
			code |= (c & 0x3F);
			if (--remain == 0){
				uchar	w;
				if (code == 0xFEFF)	continue;	// BOM
				code = unicode_to_sjis(code);
				if (code > 0x00FF){
					w = font;
				} else {
					w = font / 2;
				}
				if ((x + w) > max_width){
					if (x > width)	width = x;
					x = 0;
					y += font + LINE_SPACE;
					if (y >= LCD_HEIGHT)	break;
				}
				x += w;
			}
		}
	}
	if (x > width)	width = x;

	if (height_p != NULL){
		if (x == 0){
			*height_p = y;
		} else {
			*height_p = y + font;
		}
	}

	if (width_p != NULL){
		*width_p = width;
	}

	return width;
}


//*******************************************************
//	Shift_JIS文字列の表示に必要な領域サイズ取得
//*******************************************************
short check_sjis_area_size(char *str, uchar font, short max_width, short *width_p, short *height_p){
	ushort		code;
	uchar		c;
	uchar		remain;
	short		x, y, width;

	x = y = width = 0;
	remain = 0;
	code = 0;
	while((c = (uchar)(*str++))){
		if (c == 0){
			break;
		} else if (c == LF){	// 改行
			if (x > width)	width = x;
			x = 0;
			y += font + LINE_SPACE;
		} else if (c == CR){
			if (x > width)	width = x;
			x = 0;
		} else if (!is_sjis1(c)){
			if ((x + font / 2) > max_width){
				if (x > width)	width = x;
				x = 0;
				y += font + LINE_SPACE;
				if (y >= LCD_HEIGHT)	break;
			}
			x += font / 2;
		} else {
			str++;
			if ((x + font) > max_width){
				if (x > width)	width = x;
				x = 0;
				y += font + LINE_SPACE;
				if (y >= LCD_HEIGHT)	break;
			}
			x += font;
		}
	}
	if (x > width)	width = x;

	if (height_p != NULL){
		if (x == 0){
			*height_p = y;
		} else {
			*height_p = y + font;
		}
	}

	if (width_p != NULL){
		*width_p = width;
	}

	return width;
}


//*******************************************************
//	Shift_JIS文字列表示
//*******************************************************
void print_sjis(short x, short y, uchar font, char *str){
	ushort		c;
	uchar		w;
	int		xs;

	xs = x;
	while((c = (ushort)((uchar)*str++)) != 0){
		if (is_sjis1(c)){	// Shift_JIS

			if (*str == 0)	break;

			c = (c << 8) | ((uchar)(*str++));
			w = font;
		} else if (c == LF){	// 改行
			x = xs;
			y += font + LINE_SPACE;
			continue;
		} else {
			w = font / 2;
		}
		if ((x+w) > LCD_WIDTH){	// 折り返し
			x = xs;
			y += font + LINE_SPACE;
		}
		if (x >= LCD_WIDTH)		break;
		if (y > (LCD_HEIGHT - font))	break;
		putkj(x, y, font, c);
		x += w;
	}
}


//*******************************************************
//	半角・全角文字表示
//*******************************************************
void putkj(uchar x, uchar y, uchar font, ushort sjis){
	unsigned char	data[32];
	long			adrs;
	int				w;
	ushort			from, to;
	long			font_adrs;

	if (font == 14){
		adrs = FONT14_ADRS;
	} else if (font == 16){
		adrs = FONT16_ADRS;
	} else {
		adrs = FONT12_ADRS;
	}
	
	if (sjis >= 0x0100){
		w = font;
		adrs += 4096;	// フォントテーブル先頭
		from = 1;
		font_adrs = 0;
		while(from != 0){
			ROM_read(adrs, (uchar *)&from, 2);
			adrs += 2;
			ROM_read(adrs, (uchar *)&to, 2);
			adrs += 2;
			if (from > sjis){
				break;
			} else {
				if (to >= sjis){
					font_adrs += (sjis - from);
					ROM_read(font_adrs * font * 2 +
							 (adrs & ~4095) + 512, data, font * 2);
					break;
				} else {
					font_adrs += (to - from + 1);
				}
			}
		}
		if (from == 0)	memset(data, 0, sizeof(data));
		
	} else {
		w = font / 2;
		ROM_read(sjis * font + adrs, data, font);
	}

	// 液晶表示
	put_font(x, y, w, font, data);
}


//*********************************************************
//	UTF-8文字列を Shift_JIS文字列に変換
//	max : Shift_JIS変換後の最大文字数(最後の \0 を含まない)
//*********************************************************
void utf8_to_sjis(uchar *utf, uchar *sjis, short max){
	ushort		code;
	uchar		c;
	uchar		remain;

	remain = 0;
	code = 0;
	while(max > 0){
		c = *utf++;
		if (c == 0){
			break;
		} else if (c < 0x80){
			*sjis++ = c;
			--max;
		} else if ((c & 0xE0) == 0xC0){
			code = c & 0x1F;
			remain = 1;
		} else if ((c & 0xF0) == 0xE0){
			code = c & 0x0F;
			remain = 2;
		} else if ((c & 0xC0) == 0x80){
			code <<= 6;
			code |= (c & 0x3F);
			if (--remain == 0){
				if (code == 0xFEFF)	continue;	// BOM
				code = unicode_to_sjis(code);
				if (code > 0x00FF){
					if (max < 2)	break;
					*sjis++ = code >> 8;
					--max;
				}
				*sjis++ = (uchar)code;
				--max;
			}
		}
	}
	*sjis = 0;
}




extern PROGMEM uchar Icon[];

//*******************************************************
//	8 x 12ドット 単色アイコン表示
//*******************************************************
void put_icon(uchar x, uchar y, uchar icon, int color){
	uchar	i, j;
	uchar	data;
	PGM_VOID_P	p;

	set_window(x, y, x + 7, y + 11);

	p = &Icon[icon * 12];

	for (i = 0; i < 12; i++){
		data = pgm_read_byte(p++);
		LCD_CS_L;
		for (j = 0; j < 8; j++){
			if (data & 0x80){
				LCD_PORT = color >> 8;
				LCD_WR_L;			// WR=L
				LCD_WR_H;			// WR=H
				LCD_PORT = (uchar)color;
				LCD_WR_L;			// WR=L
				LCD_WR_H;			// WR=H
			} else {
				LCD_PORT = BGcolor >> 8;
				LCD_WR_L;			// WR=L
				LCD_WR_H;			// WR=H
				LCD_PORT = (uchar)BGcolor;
				LCD_WR_L;			// WR=L
				LCD_WR_H;			// WR=H
			}
			data <<= 1;
		}
		LCD_CS_H;
	}
}


//*********************************************************
//	Unicode -> Shift_JIS コード変換
//*********************************************************
uint unicode_to_sjis(uint uni){
	ulong	adrs;
	uint	sjis;
	uint	code;

	if (uni <= 0x0080)	return uni;

	sjis = 0;
	code = 0;

	// 上位バイトから最初の読み込みアドレス取得
	adrs = (uni >> 8);
	adrs *= 2;
	adrs += UNICODE_ADRS;
	ROM_read(adrs, (uchar *)&code, 2);
	adrs = (ulong)code + UNICODE_ADRS + 0x0200L;
	do {
		ROM_read(adrs, (uchar *)&code, 2);
		adrs += 2;
		ROM_read(adrs, (uchar *)&sjis, 2);
		adrs += 2;
		if (code == uni)	return sjis;
		if (code > uni)		break;
	} while(code != 0);

	return NO_CODE;		// ゲタマーク
}


//*********************************************************
//	MMC/SDカードからフォントファイルを読み込み
//*********************************************************
void load_font_file(void){
	ulong	rom;
	char	fname[12];
	uchar	buff[256];
	FIL		file;
	UINT	n;
	int		i;

	strcpy_P(fname, PSTR("MP3ROM.DAT"));
	if (f_open(&file, fname, FA_READ) != FR_OK)	return;

	// 全セクタプロテクト解除
	ROM_unprotect_all_sectors();
	ROM_wait();

	print_P(PSTR("\nFONT erase "));	// FONT消去
	for (i = 0; i <= (VS1053B_PATCH_ADRS >> 16); i++){
		ROM_sector_erase(i);
		putch('.');
	}

	print_P(PSTR("\nFONT load  "));	// FONT書き込み

	rom = 0;
	while(f_read(&file, buff, 256, &n) == FR_OK){
		WDR;
		if (n == 0)	break;
		ROM_write(rom, buff, 256);
		rom += 256;
		if (!(rom & 65535)){
			putch('.');
		}
		memset(buff, 0xFF, 256);
	}

	// フォント読みこみ済みフラグセット
	eeprom_write_byte(EEPROM_FONT_FLAG, 1);
}


//*********************************************************
//	Shift_JISの１文字目かどうか判断
//*********************************************************
char is_sjis1(uchar c){

	if (((c >= 0x80)&&(c <= 0x9F))||	// Shift_JIS
		((c >= 0xE0)&&(c <= 0xFC))){
		return 1;
	}
	return 0;
}


