//*******************************************************
//
//	MP3 Player in SEGA Visual Memory
//
//	ID3タグ関係
//
//	ATmega1284p
//		efuse:0xFD	BOD 2.7V
//		hfuse:0xD1	JTAG禁止, EEPROM消去保護
//		lfuse:0xE2	内蔵CR発振 8MHz
//
//	Compiler:	AVR-GCC		4.3.5
//				AVR-LIBC	1.6.8
//				binutils	2.20
//
//	2012.6.10	KAWAKAMI Yukio
//*******************************************************

#include	<avr/io.h>
#include	<avr/pgmspace.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include	"main.h"
#include	"lcd.h"
#include	"integer.h"
#include	"ff.h"
#include	"rom.h"
#include	"graph.h"
#include	"id3.h"
#include	"resource.h"


//********************************************************************
//	ファイルから ID3タグ情報取り出し
//********************************************************************
int get_id3(FIL *file, struct ID3_TAG *tag){
	uchar		buff[16];
	UINT	n;
	FRESULT	res;

	tag->title[0] = 0;
	tag->artist[0] = 0;
	//tag->album[0] = 0;

	f_lseek(file, 0);
	if ((res = f_read(file, buff, 10, &n)) != FR_OK)	return res;

	if (strncmp_P((char *)buff, PSTR("ID3"), 3) == 0){
		// ID3 バージョン2
		char	f = 0;
		char	ver;
		long	size;
		ulong	data_top;

		// ID3 バージョン2
		data_top = get_id3_long(&buff[6]) + 10;		// MP3データの先頭
		if ((data_top <= 10)||
			(data_top >= (1024L * 1024)))	return 0;

		ver = buff[3];
		while (f != 7){
			if (ver >= 3){	// Version 2.3以上。通常はこちら
				if ((res = f_read(file, buff, 10, &n)) != FR_OK)	return res;
				size = get_id3_long(&buff[4]);
				if (strncmp_P((char *)buff, PSTR("TPE1"), 4) == 0){
					get_ID3_str(file, tag->artist, size, ID3_NAME_SIZE);
					size = 0;	// 次のフレームの頭出しが済んでいるので
					f |= 1;
				} else if (strncmp_P((char *)buff, PSTR("TIT2"), 4) == 0){
					get_ID3_str(file, tag->title, size, ID3_NAME_SIZE);
					size = 0;
					f |= 2;
				//} else if (strncmp_P((char *)buff, PSTR("TALB"), 4) == 0){
				//	get_ID3_str(file, tag->album, size, ID3_NAME_SIZE);
				//	size = 0;
				//	f |= 4;
				}
			} else {	// Version 2.2以下。あるのか？
				if ((res = f_read(file, buff, 6, &n)) != FR_OK)	return res;
				size = (long)(buff[3]) << 14;
				size |= (long)(buff[4]) << 7;
				size |= (long)(buff[5]);
				if (strncmp_P((char *)buff, PSTR("TP1"), 3) == 0){
					get_ID3_str(file, tag->artist, size, ID3_NAME_SIZE);
					size = 0;	// 次のフレームの頭出しが済んでいるので
					f |= 1;
				} else if (strncmp_P((char *)buff, PSTR("TT2"), 3) == 0){
					get_ID3_str(file, tag->title, size, ID3_NAME_SIZE);
					size = 0;
					f |= 2;
				//} else if (strncmp_P((char *)buff, PSTR("TAL"), 3) == 0){
				//	get_ID3_str(file, tag->album, size, ID3_NAME_SIZE);
				//	size = 0;
				//	f |= 4;
				}
			}
			if ((buff[0] < 'A')||(buff[0] > 'Z'))	break;	// なんか変
			//if (f == 7)	break;
			if (f == 3)	break;
			if (size){
				f_lseek(file, file->fptr + size);	// 次のフレームまでスキップ
			}
			if (file->fptr >= data_top)	break;
		}
		f_lseek(file, data_top);	// 再生データの先頭へ
	} else if (strncmp_P((char *)buff, PSTR("OggS"), 4) == 0){
		// Oggタグ
	} else {
		// ID3 バージョン１かも
	}
	

	return 0;
}


//*******************************************************
//	ID3タグ longデータ取得
//*******************************************************
long get_id3_long(uchar *buff){
	long	l;

	l = (long)(*buff++) << 28;
	l |= (long)(*buff++) << 14;
	l |= (long)(*buff++) << 7;
	l |= (long)(*buff);

	return l;
}


//*******************************************************
//	ID3タグの文字列取得
//*******************************************************
int get_ID3_str(FIL *file, char *str, long size, short max){
	uchar		buff[ID3_NAME_SIZE];
	UINT	n;
	FRESULT	res;

	*str = 0;
	memset(buff, 0, sizeof(buff));

	// 1フレーム分読み込み
	if (size > (sizeof(buff) - 1)){
		if ((res = f_read(file, buff, sizeof(buff) - 1, &n)) != FR_OK)	return res;
		f_lseek(file, file->fptr + size - (sizeof(buff) - 1));
	} else {
		if ((res = f_read(file, buff, size, &n)) != FR_OK)	return res;
	}

	if (buff[0] == 0){
		// ISO-8859-1(日本の場合はたいてい Shift_JISだが、 UTF-8の可能性も)
		if (check_UTF8(&buff[1])){
			utf8_to_sjis(&buff[1], (uchar *)str, max);
		} else {
			copy_SJIS(&buff[1], (uchar *)str, max);
		}
	} else if (buff[0] == 1){	// Unicode LE BOM付き(BEの可能性あり)
		if ((buff[1] == 0xFF)&&(buff[2] == 0x00)){	// 非同期処理 + BOM
			UCS2_to_SJIS(&buff[3], (uchar *)str, max, 0);
		} else {
			UCS2_to_SJIS(&buff[1], (uchar *)str, max, 0);
		}
	} else if (buff[0] == 2){	// Unicode BE(v2.4)
		UCS2_to_SJIS(&buff[1], (uchar *)str, max, 1);
	} else if (buff[0] == 3){	// UTF-8(v2.4)
		utf8_to_sjis(&buff[1], (uchar *)str, max);
	}

	return 0;
}


//*********************************************************
//	UCS-2文字列を Shift_JIS文字列に変換
//	be : ==1 の場合、BE。ただし、BOMがある場合はそれに従う
//	max : Shift_JIS変換後の最大文字数(最後の \0 を含まない)
//*********************************************************
void UCS2_to_SJIS(uchar *ucs, uchar *sjis, short max, char be){
	ushort	code;

	while(max > 0){
		if (be){	// ビッグエンディアン
			code = ((ushort)*ucs++) << 8;
			code |= *ucs++;
		} else {	// リトルエンディアン
			code = *ucs++;
			code |= ((ushort)*ucs++) << 8;
		}
		if (code == 0){
			break;
		} else if (code == 0xFFFE){
			be ^= 1;	// エンディアンが逆だった
		} else if (code == 0xFEFF){
			continue;	// 正常な BOM
		} else {
			code = unicode_to_sjis(code);
			if (code > 0x00FF){
				if (max < 2)	break;
				*sjis++ = code >> 8;
				--max;
			}
			*sjis++ = (uchar)code;
			--max;
		}
	}
	*sjis = 0;
}


//*********************************************************
//	Shift_JIS文字列のコピー
//	max : Shift_JIS変換後の最大文字数(最後の \0 を含まない)
//*********************************************************
void copy_SJIS(uchar *src, uchar *dst, short max){
	uchar	c;

	while(((c = *src++) != 0)&&(max > 0)){
		if (is_sjis1(c)){	// Shift_JIS
			if (max < 2)	break;
			if (*src == 0)	break;
			*dst++ = c;
			*dst++ = *src++;
			max -= 2;
		} else {
			*dst++ = c;
			--max;
		}
	}
	*dst = 0;
}


//*********************************************************
//	UTF-8文字列かどうか判断
//	ただし、UCS-2の範囲のみ
//*********************************************************
char check_UTF8(uchar *str){
	uchar	c;
	uchar	code;
	uchar	remain;

	if ((str[0] == 0xEF)&&	// 先頭が BOMの場合は UTF-8
		(str[1] == 0xBB)&&
		(str[2] == 0xBF)){
		return 1;
	}

	code = 0;
	remain = 0;
	while((c = *str++) != 0){
		if (remain == 0){
			if (((c & 0xE0) == 0xC0)&&
				(c >= 0xC2)){			// UTF-8 80-7FF
				remain = 1;
			} else if ((c & 0xF0) == 0xE0){	// UTF-8 800-FFFF
				remain = 2;
			} else if (c >= 0x80){
				return 0;				// Not UTF-8
			}
		} else {
			if ((c & 0xC0) != 0x80)	return 0;	// Not UTF-8
			if (--remain == 0){
				code = 1;
			}
		}
	}
	if (remain > 0) code = 0;

	return code;
}


