//*******************************************************
//
//	MP3 Player in SEGA Visual Memory
//
//	JPEGビューア部分
//
//	ATmega1284p
//		efuse:0xFD	BOD 2.7V
//		hfuse:0xD1	JTAG禁止, EEPROM消去保護
//		lfuse:0xE2	内蔵CR発振 8MHz
//
//	Compiler:	AVR-GCC		4.3.5
//				AVR-LIBC	1.6.8
//				binutils	2.20
//
//	2012.6.22	KAWAKAMI Yukio
//*******************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>
#include	<avr/eeprom.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include	"main.h"
#include	"lcd.h"
#include	"ff.h"
#include	"rom.h"
#include	"graph.h"
#include	"jpeg.h"
#include	"vs1053.h"
#include	"resource.h"

int		JpegX, JpegY;

ushort	JpegP;
ushort	JpegN = 0;

extern ushort	JobStat;
extern uchar	SwFlag;

//*********************************************************
//	JPEG表示
//*********************************************************
void job_jpeg(void){
	FILINFO	file;
	static uchar	last_sw;
	uchar	sw;
	char	rc;

	switch(JobStat & 0x00FF){
	  case 0:
		cls();
		if (JpegN == 0){
			JpegN = search_jpeg_file(9999, &file);
			JpegP = 1;
			if (JpegN == 0){
				locate(12, 8);
				print_P(PSTR("No Jpeg File"));
			}
		}
		if (!SwFlag){
			JobStat++;
		}
		break;

	  case 1:
		if (JpegP > JpegN)	JpegP = 1;
		search_jpeg_file(JpegP, &file);
		rc = load_jpeg(file.fname);
		if ((rc == 0)||(rc == 1)){
			char	msg[20];
			itoa(JpegP, msg, 10);
			locate(12, 8);
			print(msg);
			putch('/');
			itoa(JpegN, msg, 10);
			print(msg);
			if (rc == 1)	beep(1, BEEP_HIGH);
		} else {
			char	msg[20];
			locate(12, 8);
			print(file.fname);
			sprintf_P(msg, PSTR("\n  Error: %d"), rc);
			print(msg);
		}
		JobStat++;
		break;

	  case 2:
		sw = SwFlag & (~last_sw);
		if (sw & SW_POWER){
			beep(1, BEEP_HIGH);
			power_off();
		} else if (sw & SW_LEFT){
			beep(1, BEEP_HIGH);
			if (JpegP == 1){
				JpegP = JpegN;
			} else {
				--JpegP;
			}
			--JobStat;
		} else if (sw & SW_RIGHT){
			beep(1, BEEP_HIGH);
			if (++JpegP > JpegN){
				JpegP = 1;
			}
			--JobStat;
		} else if (sw & SW_MODE){
			beep(1, BEEP_HIGH);
			JobStat = JOB_MP3;
		}
		break;

	  default:
		JobStat = JOB_JPEG;
		break;
	}

	last_sw = SwFlag;
}


//********************************************************************
//	JPEGファイルを探す
//********************************************************************
ushort search_jpeg_file(ushort num, FILINFO *file){
	DIR	dir;
	short	n = 0;
	char	path;

	path = 0;	// root
	if (f_opendir(&dir, &path) != FR_OK)	return 0;

	while((f_readdir(&dir, file) == FR_OK)&&
		  (file->fname[0] != 0)){
		char	*p;
		if (file->fname[0] == '.')	continue;
		if (file->fattrib & (AM_DIR | AM_VOL))	continue;
		p = strchr(file->fname, '.');
		if ((p != NULL)&&(strcmp_P(p, PSTR(".JPG")) == 0)){
			n++;
			if (--num == 0)	break;
		}
	}
	
	return n;
}


//*********************************************************
//	TJpegDec データ入力
//*********************************************************
UINT tjd_input (
	JDEC* jd,		/* Decompression object */
	BYTE* buff,		/* Pointer to the read buffer (NULL:skip) */
	UINT nd			/* Number of bytes to read/skip from input stream */
)
{
	UINT rb;
	FIL *fil = (FIL*)jd->device;	/* Input stream of this session */

	if (buff) {	/* Read nd bytes from the input strem */
		f_read(fil, buff, nd, &rb);
		return rb;	/* Returns number of bytes could be read */

	} else {	/* Skip nd bytes on the input stream */
		return (f_lseek(fil, f_tell(fil) + nd) == FR_OK) ? nd : 0;
	}
}



//*********************************************************
//	TJpegDec 画面出力
//*********************************************************
UINT tjd_output (
	JDEC* jd,		/* Decompression object of current session */
	void* bitmap,	/* Bitmap data to be output */
	JRECT* rect		/* Rectangular region to output */
)
{
	int		n;
	uchar	*p;
	uchar	data;
	jd = jd;	/* Suppress warning (device identifier is not needed in this appication) */

	WDR;	// 展開に時間がかかるので必須

	if ((SwFlag & SW_B)||(SwFlag & SW_RIGHT))	return 0;	// 途中停止
	
	set_window(rect->left + JpegX, rect->top + JpegY, rect->right + JpegX, rect->bottom + JpegY);

	n = (rect->right - rect->left + 1) * (rect->bottom - rect->top + 1);
	p = (uchar *)bitmap;

	LCD_CS_L;
	while(n > 0){
		data = *p++;
		LCD_PORT = *p++;
		LCD_WR_L;
		LCD_WR_H;
		LCD_PORT = data;
		LCD_WR_L;
		LCD_WR_H;
		--n;
	}
	LCD_CS_H;

	return 1;	/* Continue to decompression */
}


//*********************************************************
//	JPEGファイル読み込み表示
//*********************************************************
char load_jpeg (char *fname){
	JDEC	jd;		/* Decompression object (70 bytes) */
	JRESULT rc;
	BYTE	scale;
	FIL		file;
	char	work[3100];


	if (f_open(&file, fname, FA_READ)) return -1;

	cls();

	/* Prepare to decompress the file */
	rc = jd_prepare(&jd, tjd_input, work, sizeof(work), &file);
	if (rc == JDR_OK) {
		if ((jd.width > 120 * 8)||(jd.height > 120 * 8)){
			rc = 9;	// over size
		} else {

			/* Determine scale factor */
			for (scale = 0; scale < 3; scale++) {
				if ((jd.width >> scale) <= 120 && (jd.height >> scale) <= 120) break;
			}

			// 画面中央センタリング
			JpegX = (120 - (jd.width >> scale)) / 2 + 8;
			JpegY = (120 - (jd.height >> scale)) / 2 + 8;

			/* Start to decompress the JPEG file */
			rc = jd_decomp(&jd, tjd_output, scale);	/* Start to decompress */
		}

	}

	return rc;
}


