//*******************************************************
//	LCD表示
//	ZY-FGD1442701V1	16bit color mode
//*******************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>
#include	<avr/eeprom.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include "main.h"
#include "lcd.h"
#include "LCDfont.h"

unsigned char	CurX, CurY;
ushort			Color, BGcolor;

//*******************************************************
//	フォントデータを描画
//	フォントデータは FONTX2形式の生データ
//	描画色は Color、背景はBGcolor
//*******************************************************
void put_font(short xs, short y, short width, short height, uchar *font_buff){
	uchar	i, data = 0, mask;

	set_window(xs, y, xs + width - 1, y + height - 1);

	while(height > 0){
		mask = 0;
		LCD_CS_L;			// CS=L
		for (i = 0; i < width; i++){
			if (!mask){
				mask = 0x80;
				data = *font_buff++;
			}
			if (data & mask){
				LCD_PORT = Color >> 8;
				LCD_WR_L;			// WR=L
				LCD_WR_H;			// WR=H
				LCD_PORT = (uchar)Color;
				LCD_WR_L;			// WR=L
				LCD_WR_H;			// WR=H
			} else {
				LCD_PORT = BGcolor >> 8;
				LCD_WR_L;			// WR=L
				LCD_WR_H;			// WR=H
				LCD_PORT = (uchar)BGcolor;
				LCD_WR_L;			// WR=L
				LCD_WR_H;			// WR=H
			}
			mask >>= 1;
		}
		LCD_CS_H;			// CS=H
		--height;
	}
}


//*******************************************************
//	矩形塗りつぶし
//*******************************************************
void fill_rect(uchar xs, uchar ys, uchar xe, uchar ye, ushort color){
	int		n;

	set_window(xs, ys, xe, ye);
	n = (xe - xs + 1) * (ye - ys + 1);

	LCD_CS_L;			// CS=L
	while(n > 0){
		LCD_PORT = color >> 8;
		LCD_WR_L;			// WR=L
		LCD_WR_H;			// WR=H
		LCD_PORT = (uchar)color;
		LCD_WR_L;			// WR=L
		LCD_WR_H;			// WR=H
		--n;
	}
	LCD_CS_H;			// CS=H
}


//*******************************************************
//	矩形枠表示
//*******************************************************
void rectangle(uchar xs, uchar ys, uchar xe, uchar ye, ushort color){

	fill_rect(xs, ys, xe, ys, color);
	fill_rect(xs, ye, xe, ye, color);
	fill_rect(xs, ys+1, xs, ye-1, color);
	fill_rect(xe, ys+1, xe, ye-1, color);
}


//*******************************************************
//	文字列表示
//*******************************************************
void print(char *str){
	char	c;

	while((c = *str++)){
		putch(c);
	}
}


//*******************************************************
//	固定文字列表示
//*******************************************************
void print_P(PGM_VOID_P str){
	char	c;

	while((c = pgm_read_byte(str++))){
		putch(c);
	}
}


//*******************************************************
//	カーソル位置に指定した色の半角文字描画
//*******************************************************
void putch(char ch){

	if (ch == CR){
		CurX = 0;
	} else if (ch == LF){
		CurX = 0;
		if (CurY < LCD_HEIGHT)	CurY += 8;
	} else {
		if ((ch < ' ')||(ch > 0x7F)){
			ch = '.';
		}
		if (CurX > (LCD_WIDTH - 6)){
			CurX = 0;
			if (CurY < LCD_HEIGHT)	CurY += 8;
		}
		if (CurY < (LCD_HEIGHT - 7)){
			PGM_VOID_P	ps;
			PGM_VOID_P	p;
			uchar	i, mask;

			set_window(CurX, CurY, CurX + 5, CurY + 7);

			ps = (PGM_VOID_P)&AsciiFont[(ch - 0x20) * 5];
			for (mask = 1; mask; mask <<= 1){
				p = ps;
				for (i = 0; i < 5; i++){
					if (pgm_read_byte(p++) & mask){
						LCD_data(Color >> 8);
						LCD_data((uchar)Color);
					} else {
						LCD_data(BGcolor >> 8);
						LCD_data((uchar)BGcolor);
					}
				}
				LCD_data(BGcolor >> 8);
				LCD_data((uchar)BGcolor);
			}
			CurX += 6;
		}
	}
}


//*******************************************************
//	カーソル位置設定
//*******************************************************
void locate(uchar x, uchar y){

	CurX = x;
	CurY = y;
}


//*******************************************************
//	文字色設定
//*******************************************************
void color(ushort color){

	Color = color;
}


//*******************************************************
//	文字の背景色設定
//*******************************************************
void back_color(ushort color){

	BGcolor = color;
}


//*******************************************************
//	LCD コマンド送信
//*******************************************************
void LCD_command(uchar data){

	LCD_COMMAND;		//
	LCD_CS_L;			// CS=L
	LCD_PORT = data;
	LCD_WR_L;			// WR=L
	LCD_WR_H;			// WR=H
	LCD_CS_H;			// CS=H
}


//*******************************************************
//	LCD データ送信
//*******************************************************
void LCD_data(uchar data){

	LCD_DATA;		//
	LCD_CS_L;			// CS=L
	LCD_PORT = data;
	LCD_WR_L;			// WR=L
	LCD_WR_H;			// WR=H
	LCD_CS_H;			// CS=H
}


//*******************************************************
//	LCD初期化
//*******************************************************
void LCD_init(void){

	LCD_RES_L;
	wait_timer(5);
	LCD_RES_H;

	wait_timer(12);

	LCD_command(SWRESET);
	wait_timer(5);
	LCD_command(SLPOUT);

	LCD_command(VCOM4L);
	LCD_data(0x40);
	LCD_data(0x03);
	LCD_data(0x1A);

	LCD_command(NVCTR1);
	LCD_data(0x60);
	LCD_command(VMOFCTR);
	LCD_data(0x10);

	LCD_command(FRMCTR1);
	LCD_data(0x04);
	LCD_data(0x25);
	LCD_data(0x18);

	LCD_command(FRMCTR2);
	LCD_data(0x04);
	LCD_data(0x25);
	LCD_data(0x18);

	LCD_command(FRMCTR3);
	LCD_data(0x04);
	LCD_data(0x25);
	LCD_data(0x18);
	LCD_data(0x04);
	LCD_data(0x25);
	LCD_data(0x18);

	LCD_command(INVCTR);
	LCD_data(0x03);

	LCD_command(DISSET5);
	LCD_data(0x15);
	LCD_data(0x02);

	LCD_command(PWCTR1);
	LCD_data(0x02);
	LCD_data(0x70);

	LCD_command(PWCTR2);
	LCD_data(0x07);

	LCD_command(PWCTR3);
	LCD_data(0x01);
	LCD_data(0x01);

	LCD_command(PWCTR4);
	LCD_data(0x02);
	LCD_data(0x07);

	LCD_command(PWCTR5);
	LCD_data(0x02);
	LCD_data(0x04);

	LCD_command(PWCTR6);
	LCD_data(0x11);
	LCD_data(0x17);

	LCD_command(VMCTR1);
	LCD_data(0x3C);
	LCD_data(0x4F);

	// MADCTLのビット
	// MY MX MV ML RGB MH 0 0
	LCD_command(MADCTL);
	//LCD_data(0xC8);			// 180度回転 (CASET, RASETの調整必要)
	//LCD_data(0x08);		// コネクタ側を上に通常表示
	LCD_data(0xA8);			// コネクタを右側 

	LCD_command(COLMOD);
	//LCD_data(0x03);			// 12bit/pixel
	LCD_data(0x05);				// 16bit/pixel

	LCD_command(GAMCTRP1);
	LCD_data(0x06);
	LCD_data(0x0E);
	LCD_data(0x05);
	LCD_data(0x20);
	LCD_data(0x27);
	LCD_data(0x23);
	LCD_data(0x1C);
	LCD_data(0x21);
	LCD_data(0x20);
	LCD_data(0x1C);
	LCD_data(0x26);
	LCD_data(0x2F);
	LCD_data(0x00);
	LCD_data(0x03);
	LCD_data(0x00);
	LCD_data(0x24);

	LCD_command(GAMCTRN1);
	LCD_data(0x06);
	LCD_data(0x10);
	LCD_data(0x05);
	LCD_data(0x21);
	LCD_data(0x27);
	LCD_data(0x22);
	LCD_data(0x1C);
	LCD_data(0x21);
	LCD_data(0x1F);
	LCD_data(0x1D);
	LCD_data(0x27);
	LCD_data(0x2F);
	LCD_data(0x05);
	LCD_data(0x03);
	LCD_data(0x00);
	LCD_data(0x3F);

	LCD_command(DISPON);

	Color = COLOR_WHITE;
	BGcolor = COLOR_BLACK;
}


//*******************************************************
//	LCDクリア
//*******************************************************
void cls(void){

	CurX = CurY = 0;
	fill_rect(0, 0, LCD_WIDTH-1, LCD_HEIGHT-1, BGcolor);
}


//*******************************************************
//	LCD Window set
//*******************************************************
void set_window(uchar xs, uchar ys, uchar xe, uchar ye){

	LCD_command(CASET);
	LCD_data(0);
	LCD_data(xs + 3);
	LCD_data(0);
	LCD_data(xe + 3);

	LCD_command(RASET);
	LCD_data(0);
	LCD_data(ys + 2);
	LCD_data(0);
	LCD_data(ye + 2);

	LCD_command(RAMWR);
	LCD_DATA;
}


