#ifndef	__LCD_H__
#define	__LCD_H__
//*******************************************************
//	LCD表示
//	ZY-FGD1442701V1	16bit color mode
//*******************************************************

#include "main.h"

#define	LCD_WIDTH	128
#define	LCD_HEIGHT	128


#define	COLOR_WHITE		0xFFFF
#define	COLOR_RED		0xF800
#define	COLOR_GREEN		0x07E0
#define	COLOR_BLUE		0x001F
#define	COLOR_YELLOW	0xFFE0
#define	COLOR_CYAN		0x07FF
#define	COLOR_MAGENTA	0xF81F

#define	RGB(r,g,b)	((((ushort)r & 0xF8) << 11)|(((ushort)g & 0xFC) << 5)|((ushort)b >> 3))

#define	COLOR_BLACK		0


// ST7735 COMMAND
//#define	NOP		0x00	// No Operation
#define	SWRESET	0x01	// Software reset
#define	SLPIN	0x10	// Sleep in & booster off
#define	SLPOUT	0x11	// Sleep out & booster on
#define	PTLON	0x12	// Partial mode on
#define	NORON	0x13	// Partial off (Normal)
#define	INVOFF	0x20	// Display inversion off
#define	INVON	0x21	// Display inversion on
#define	GAMSET	0x26	// Gamma curbe select
#define	DISPOFF	0x28	// Display off
#define	DISPON	0x29	// Display on
#define	CASET	0x2A	// Column addrss set
#define	RASET	0x2B	// Row address set
#define	RAMWR	0x2C	// Memory write
#define	PTLAR	0x30	// partial start/end address set
#define	TEOFF	0x34	// Tearing effect line off
#define	TEON	0x35	// Tearing effect mode set & on
#define	MADCTL	0x36	// Memory data access control
#define	IDMOFF	0x38	// Idle mode off
#define	IDMON	0x39	// Idle mode on
#define	COLMOD	0x3A	// Interface pixel format

#define	FRMCTR1	0xB1	// In normal mode (Full colors)
#define	FRMCTR2	0xB2	// In Idle mode (8-colors)
#define	FRMCTR3	0xB3	// In partial mode + Full colors
#define	INVCTR	0xB4	// Display inversion control
#define	DISSET5	0xB6	// Display function setting
#define	PWCTR1	0xC0	// Power control setting
#define	PWCTR2	0xC1	// Power control setting
#define	PWCTR3	0xC2	// In normal mode (Full colors)
#define	PWCTR4	0xC3	// In Idle mode (8-colors)
#define	PWCTR5	0xC4	// In pertioal mode + Full colors
#define	VMCTR1	0xC5	// VCOM control 1
#define	VMOFCTR	0xC7	// Set VCOM offset control
#define	WRID2	0xD1	// Set LCM version code
#define	WRID3	0xD2	// Customer Project code
#define	PWCTR6	0xFC	// In partial mode + Idle
#define	NVCTR1	0xD9	// EEPROM control status
#define	NVCTR2	0xDE	// EEPROM Read Command
#define	NVCTR3	0xDF	// EEPROM Write Command
#define	GAMCTRP1	0xE0	// Gamma adjustment (+ polarity)
#define	GAMCTRN1	0xE1	// Gamma adjustment (- polarity)
#define	EXTCTRL	0xF0	// Extenction Command Control
#define	VCOM4L	0xFF	// Vcom 4 Level control


#define	LCD_WR_H	PORTD |= (1<<LWR)
#define	LCD_WR_L	PORTD &= ~(1<<LWR)
#define	LCD_RD_H	PORTD |= (1<<LRD)
#define	LCD_RD_L	PORTD &= ~(1<<LRD)
#define	LCD_CS_H	PORTD |= (1<<LCS)
#define	LCD_CS_L	PORTD &= ~(1<<LCS)
#define	LCD_RES_H	PORTD |= (1<<LRES)
#define	LCD_RES_L	PORTD &= ~(1<<LRES)
#define	LCD_DATA	PORTD |= (1<<LCD)
#define	LCD_COMMAND	PORTD &= ~(1<<LCD)
#define	LCD_PORT	PORTC
#define	LCD_DDR		DDRC

void LCD_command(uchar data);
void LCD_data(uchar data);

void LCD_init(void);
void set_dot(uchar x, uchar y, ushort color);
void set_window(uchar xs, uchar ys, uchar xe, uchar ye);
void fill_rect(uchar xs, uchar ys, uchar xe, uchar ye, ushort color);
void rectangle(uchar xs, uchar ys, uchar xe, uchar ye, ushort color);

void locate(uchar x, uchar y);
void color(ushort color);
void back_color(ushort color);

void putch(char ch);
void print(char *str);
void print_P(PGM_VOID_P str);
void cls(void);
void put_font(short xs, short y, short width, short height, uchar *font_buff);

#endif	// __LCD_H__
