//*******************************************************
//
//	MP3 Player in SEGA Visual Memory
//
//	ATmega1284p
//		efuse:0xFD	BOD 2.7V
//		hfuse:0xD1	JTAG禁止, EEPROM消去保護
//		lfuse:0xE2	内蔵CR発振 8MHz
//
//	Compiler:	AVR-GCC		4.3.5
//				AVR-LIBC	1.6.8
//				binutils	2.20
//
//	2012.6.16	KAWAKAMI Yukio
//*******************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>
#include	<avr/eeprom.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include	"main.h"
#include	"com.h"
#include	"lcd.h"
#include	"ff.h"
#include	"rom.h"
#include	"graph.h"
#include	"vs1053.h"
#include	"mp3.h"
#include	"id3.h"
//#include	"tjpgd.h"
#include	"resource.h"

#include	<util/delay.h>

COM_WORK	Com0, Com1;

FATFS	FatFs;

volatile uchar	Timer;
volatile uchar	WTimer;
volatile uchar	SwTimer;

volatile int	BattVolt;

volatile uchar	SwFlag;


extern volatile uchar	Volume;			// 音量
extern volatile uchar	VolumeReg;		// 現在の実際の音量設定値
extern volatile uchar	BeepTimer;		// Beep音の長さ
extern volatile uchar	BusyTimer;
extern volatile uchar	MuteTimer;
extern volatile uchar	DecodeTimer;
extern volatile uchar	VolumeTimer;

extern ulong		MusicP;		// 再生中の曲のポインタ
extern ushort		PlayMusic;	// 再生中の曲番号
extern ulong		Music1Size;	// カードの１曲目のファイルサイズ

ushort	JobStat;

extern ushort			Color, BGcolor;

//*******************************************************
//	タイマー０比較一致Ａ割り込み
//	100Hz
//*******************************************************
ISR(TIMER0_COMPA_vect){

	if (Timer)		--Timer;
	if (WTimer)		--WTimer;
	if (SwTimer)	--SwTimer;

	if (BeepTimer)	--BeepTimer;
	if (BusyTimer)	--BusyTimer;
	if (MuteTimer)	--MuteTimer;
	if (DecodeTimer)	--DecodeTimer;
	if (VolumeTimer)	--VolumeTimer;

	// スイッチ入力チェック
	if (PORTD & (1<<LCS)){	// 液晶操作中はスイッチ入力検出しない
		PORTC = 0xFF;		// プルアップ抵抗
		DDRC = 0;			// 入力ポート設定
		NOP;
		NOP;
		NOP;
		NOP;
		SwFlag = ~ PINC;	// LCD_DBと共用していることに注意
		PORTC = 0;
		DDRC = 0xFF;
	}

	// A/D変換開始
	ADCSRA |= (1<<ADIE);
	ADCSRA |= (1<<ADSC);
}


//*******************************************************
//	A/D変換
//*******************************************************
ISR(ADC_vect){

	BattVolt = (int)ADCL;
	BattVolt |= (((int)ADCH & 3) << 8);
	ADCSRA &= ~(1<<ADEN);	// A/D停止
}


//*******************************************************
//	メインルーチン
//*******************************************************
int main(void){
	uchar	i;

	cli();

	PORTA = (1<<CDET)|(1<<XRES)|(1<<PLUG);
	DDRA = (1<<BK1)|(1<<BK0)|(1<<XRES)|(1<<POWER)|(1<<AMP);

	PORTB = (1<<MOSI)|(1<<CCS)|(1<<XDCS)|(1<<XCS);
	DDRB = (1<<SCK)|(1<<MOSI)|(1<<CCS)|(1<<XDCS)|(1<<XCS);

	PORTC = 0;
	DDRC = 0xFF;

	PORTD = (1<<LRES)|(1<<LWR)|(1<<LRD)|(1<<LCS)|(1<<MCS);
	DDRD = (1<<LRES)|(1<<LWR)|(1<<LRD)|(1<<LCD)|(1<<LCS)|(1<<MCS);

	if (MCUSR & (1<<WDRF)){
		// ウォッチドッグリセット
	}
	MCUSR = 0;

	WDR;
	WDTCSR = (1<<WDCE)|(1<<WDE);	// WDCEとWDEを1にしてタイマー変更をする
	WDTCSR = (1<<WDE)|(1<<WDP2)|(1<<WDP1)|(1<<WDP0);	// ウォッチドッグタイマーセット 約２秒
	
	TCCR0A = (1<<WGM01);			// CTC動作
	TCCR0B = (1<<CS02)|(1<<CS00);	// プリスケーラ 1/1024
	OCR0A = F_CPU/1024/100 - 1;		// タイマカウント 100Hz
	TIMSK0 = (1<<OCIE0A);			// 比較一致Ａ割り込み許可

	ADMUX = (1<<REFS1)|VOLT;	// 内部基準電圧 1.1V, A/D ch
	ADCSRA = (1<<ADEN)|(1<<ADIE)|(1<<ADPS2)|(1<<ADPS1);	// A/D変換回路ON プリスケーラ 8MHz/64 = 125kHz
	ADCSRB = 0;
	DIDR0 = (1<<ADC7D);

	SMCR = (1<<SE);			// スリープ許可 アイドルモード

	// TWI電力カット
	PRR0 = (1<<PRTWI);
	ACSR = (1<<ACD);			// アナログコンパレータ禁止

	SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR1);	// SPIマスタ Mode 0
	SPSR = (1<<SPI2X);						// fosc/32 この段階ではロースピード

	for (i = 0; i < 10; i++){
		SPI_in();
	}

	sei();

	wait_timer(10);

	PORTA |= (1<<POWER);	// 電源保持

	LCD_init();
	cls();
	put_grp(8, 6, GRP_ONPU);
	locate(32, 16);
	print_P(PSTR("MP3 Player\n"));
	print_P(PSTR("   in Visual Memory\n"));
	print_P(PSTR("\n"));
	print_P(PSTR(" Version " VERSION "\n"));
	print_P(PSTR(" "__DATE__ " " __TIME__));
	put_grp(48, 64, GRP_CAT);
	locate(34, 108);
	print_P(PSTR("yuki-lab.jp"));

	PORTA |= (1<<BK1);
	PORTA |= (1<<BK0);

	MP3_init();
	amp_mute(ON);

	Volume = eeprom_read_byte(EEPROM_VOLUME);
	if (Volume == 0xFF)	Volume = 0x40;
	VolumeReg = 0xFF;

	PlayMusic = eeprom_read_word(EEPROM_MUSIC);
	eeprom_read_block(&MusicP, EEPROM_MUSIC_P, 4);
	eeprom_read_block(&Music1Size, EEPROM_SIZE, 4);
	if (PlayMusic == 0xFFFF){
		PlayMusic = 0;
		MusicP = 0;
		Music1Size = 0;
	}

	// 電源ボタンが放されるまで待つ
	while(SwFlag & SW_POWER){
		wait_timer(1);
	}

	f_mount(0, &FatFs);

	// フォントが読み込まれていないならロード
	if (eeprom_read_byte(EEPROM_FONT_FLAG) == 0xFF){
		load_font_file();	// SDカードからフォントファイル読み込み
	}

	load_resource();	// リソースファイル読み込み

	set_jack_mute();	// ヘッドフォンジャックの状態に応じてアンプミュート
	
	beep(1, BEEP_HIGH);

	JobStat = 0;

	i = 0;
	for (;;){
		WDR;
		SLEEP;

		switch(JobStat & 0xFF00){
		  case JOB_INIT:
			job_init();
			break;
		  case JOB_MP3:
			job_mp3();
			break;
		  case JOB_JPEG:
			job_jpeg();
			break;
		  default:
			break;
		}

		print_batt(0);
		print_vol(0);
		if (check_batt() <= 310){	// 3.10V以下
			mp3_close();
			beep(4, BEEP_LOW);
			power_off();
		}

		set_jack_mute();	// ヘッドフォンジャックの状態に応じてアンプミュート
		job_vs1053();
	}
}


//*******************************************************
//	初期状態
//*******************************************************
void job_init(void){

	switch(JobStat & 0x00FF){
	  case 0:
		color(COLOR_WHITE);
		back_color(COLOR_BLACK);
		//load_jpeg("BETSU.JPG");
		JobStat++;
		break;
	  case 1:
		//if (SwFlag & SW_A){
			JobStat = JOB_MP3;
		//}
		break;
	  default:
		JobStat = 0;
		break;
	}
}


//*********************************************************
//	音量表示
//*********************************************************
void print_vol(char view){

	PROGMEM static uchar bit[] = { 0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 5, 5 };
	static uchar	old_vol = 0;

	uchar	vol;
	uchar	i;
	int		color;

	if (view)	old_vol = 0;

	vol = Volume;
	if (vol == old_vol)	return;
	old_vol = vol;
	
	put_icon(60, 8, 0, COLOR_WHITE);	// スピーカーアイコン

	color = COLOR_MAGENTA;
	for (i = 0; i < 14; i++){
		if (vol < 0x58){
			vol += 0x04;
		} else {
			color = BGcolor;
		}
		fill_rect(64 + 7 + i,
					5 - pgm_read_byte(&bit[i]) + 8,
					64 + 7 + i,
					5 + pgm_read_byte(&bit[i]) + 8,
					color);
	}
}


//*******************************************************
//	バッテリーグラフ表示
//*******************************************************
void print_batt(char view){
	static int	old_batt = 0;
	int		b;
	int		v;

	v = check_batt();

	if (view)	old_batt = 0;
	
	if (v != old_batt){
		int	color;
		b = old_batt = v;

		b -= 300;				// 3.00V最小
		if (b < 0)	b = 0;
		if (b > 120)	b = 120;		// 3.00 + 1.20 = 4.20V max
		if (b <= 50){			// 3.50V以下
			color = COLOR_RED;
		} else {
			color = COLOR_CYAN;
		}
		b /= 5;

		fill_rect(100, 2+8, 101, 3+8, BGcolor);
		fill_rect(100, 4+8, 101, 9+8, COLOR_WHITE);
		fill_rect(100, 10+8, 101, 11+8, BGcolor);
		rectangle(102, 2+8, 127, 11+8, COLOR_WHITE);

		if (b){
			fill_rect(127 - b, 3+8, 126, 10+8, color);
		}
		if (b < 24){
			fill_rect(103, 3+8, 126 - b, 10+8, COLOR_BLACK);
		}
	}
}


//*******************************************************
//	電源OFF
//*******************************************************
void power_off(void){

	PORTA &= ~(1<<AMP);		// AMP OFF

	// バックライト OFF
	PORTA &= ~(1<<BK1);
	PORTA &= ~(1<<BK0);

	// 設定値保存
	if (eeprom_read_byte(EEPROM_VOLUME) != Volume){
		eeprom_write_byte(EEPROM_VOLUME, Volume);
	}
	eeprom_write_word(EEPROM_MUSIC, PlayMusic);
	eeprom_write_block(&MusicP, EEPROM_MUSIC_P, 4);
	eeprom_write_block(&Music1Size, EEPROM_SIZE, 4);

	PORTA &= ~(1<<POWER);	// 電源OFF
	for(;;){
		WDR;
		SLEEP;
	}
}


//*******************************************************
//	電池電圧取得(0.01V単位)
//*******************************************************
int check_batt(void){
	long	l;

	cli();
	l = (long)BattVolt;
	sei();

	//l *= (100 + 33);		// 100kΩ, 33kΩ分圧
	//l /= 33;
	//l *= 110;			// 1.1V基準電圧
	//l /= 1023;

	l *= 111;
	l /= 256;

	return (int)l;
}


//*******************************************************
//	10ms単位時間待ち
//*******************************************************
void wait_timer(uchar tm){

	WTimer = tm;

	while(WTimer){
		WDR;
		SLEEP;
	}
}


//*******************************************************
//	SPI入力
//*******************************************************
uchar SPI_in(void){

	SPDR = 0xFF;
	while(!(SPSR & (1<<SPIF)));
	return SPDR;
}


//*******************************************************
//	SPI出力
//*******************************************************
void SPI_out(uchar data){

	SPDR = data;
	while(!(SPSR & (1<<SPIF)));
}


