#ifndef	__MAIN_H__
#define	__MAIN_H__
//*******************************************************
//
//	MP3 Player in SEGA Visual Memory
//
//	ATmega1284p
//		efuse:0xFD	BOD 2.7V
//		hfuse:0xD1	JTAG禁止, EEPROM消去保護
//		lfuse:0xE2	内蔵CR発振 8MHz
//
//	Compiler:	AVR-GCC		4.3.5
//				AVR-LIBC	1.6.8
//				binutils	2.20
//
//	2012.6.16	KAWAKAMI Yukio
//*******************************************************

#include	<avr/pgmspace.h>

#define		VERSION		"1.0.0"

#define		F_CPU	8000000UL

#define		CR	13
#define		LF	10
#define		TAB	9
#define		ON	1
#define		OFF	0
#define		TRUE	1
#define		FALSE	0

#define		NOP		asm volatile("nop\n"::)
#define		WDR		asm volatile("wdr\n"::)
#define		SLEEP	asm volatile("sleep\n"::)

typedef	unsigned char	uchar;
typedef	unsigned int	uint;
typedef	unsigned long	ulong;
typedef unsigned short	ushort;

#define		VOLT	7		// PORTA
#define		BK1		6
#define		BK0		5
#define		CDET	4
#define		XRES	3
#define		PLUG	2
#define		POWER	1
#define		AMP		0

#define		SCK		7		// PORTB
#define		MISO	6
#define		MOSI	5
#define		CCS		4
#define		BEEP	3
#define		DREQ	2
#define		XDCS	1
#define		XCS		0

#define		DB7		7		// PORTC
#define		DB6		6
#define		DB5		5
#define		DB4		4
#define		DB3		3
#define		DB2		2
#define		DB1		1
#define		DB0		0

#define		LRES	7		// PORTD
#define		LWR		6
#define		LRD		5
#define		LCD		4
#define		LCS		3
#define		MCS		2
#define		TXD		1
#define		RXD		0

#define		SW_UP		(1<<0)
#define		SW_LEFT		(1<<1)
#define		SW_RIGHT	(1<<2)
#define		SW_DOWN		(1<<3)
#define		SW_POWER	(1<<4)
#define		SW_MODE		(1<<5)
#define		SW_A		(1<<6)
#define		SW_B		(1<<7)


typedef struct tagGRPHEADER {
	unsigned short	mark;
	short	width;
	short	height;
	short	type;
	short	comp1;
	short	comp2;
	unsigned char yobi[20];
} GRPHEADER;



#define	EEPROM_SELF_PROGRAM	(uint8_t *)0
#define	EEPROM_FONT_FLAG	(uint8_t *)1
#define	EEPROM_VOLUME		(uint8_t *)2

#define	EEPROM_MUSIC		(uint16_t *)4
#define	EEPROM_MUSIC_P		(uint32_t *)6
#define	EEPROM_SIZE			(uint32_t *)10


#define	JOB_INIT			0x0000
#define	JOB_MP3				0x1000
#define	JOB_JPEG			0x2000

void job_init(void);

uchar SPI_in(void);
void SPI_out(uchar data);

void print_batt(char view);
void print_vol(char view);
int check_batt(void);
void power_off(void);

void wait_timer(uchar tm);

#endif	// __MAIN_H__
