//*******************************************************
//
//	MP3 Player in SEGA Visual Memory
//
//	MP3再生処理
//
//	ATmega1284p
//		efuse:0xFD	BOD 2.7V
//		hfuse:0xD1	JTAG禁止, EEPROM消去保護
//		lfuse:0xE2	内蔵CR発振 8MHz
//
//	Compiler:	AVR-GCC		4.3.5
//				AVR-LIBC	1.6.8
//				binutils	2.20
//
//	2012.6.15	KAWAKAMI Yukio
//******************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>
#include	<avr/eeprom.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include	"main.h"
#include	"lcd.h"
#include	"graph.h"
#include	"ff.h"
#include	"rom.h"
#include	"vs1053.h"
#include	"mp3.h"
#include	"id3.h"
#include	"resource.h"

FIL	Mp3;

extern uchar	VS1053stat;
extern volatile uchar	Volume;			// 音量

extern volatile uchar	BusyTimer;
extern volatile uchar	MuteTimer;
volatile uchar	DecodeTimer;
volatile uchar	VolumeTimer;

extern volatile uchar	SwFlag;
extern ushort	JobStat;

struct ID3_TAG	Tag;
ulong		MusicP;		// 再生中の曲のポインタ
ushort		PlayMusic;	// 再生中の曲番号
ulong		Music1Size;	// カードの１曲目のファイルサイズ
ushort		MusicCount;	// カード中の曲数

extern ushort			Color, BGcolor;

//********************************************************************
//	MP3再生
//********************************************************************
char job_mp3(void){
	static char		pause_flag = 0;
	static uchar	last_sw = 0;
	static ushort	last_prog = 0;
	FILINFO	file;
	ulong	size1;
	uchar	sw;
	ushort	prog;
	char	ret = 0;

	switch(JobStat & 0x00FF){
	  case 0:
		cls();
		put_grp(8, 6, GRP_ONPU);
		print_vol(1);
		print_batt(1);
		rectangle(LCD_WIDTH - BAR_GRAPH_MAX - 2, LCD_HEIGHT - 6, LCD_WIDTH- 1, LCD_HEIGHT - 1, COLOR_BLUE);

		if ((MusicCount = search_sound_file(9999, &file))){
			// １曲目のサイズチェック
			search_sound_file(1, &file);
			size1 = file.fsize;
			if (size1 != Music1Size){
				// カード交換されたので初期化
				Music1Size = size1;
				MusicP = 0;
				PlayMusic = 1;
			}
			// 再生開始
			//put_grp_tp(92, LCD_HEIGHT - 24, GRP_PLAY_ON);

			if (!PlayMusic ||(PlayMusic > MusicCount)){
				PlayMusic = 1;
				MusicP = 0;
			}
			search_sound_file(PlayMusic, &file);
			mp3_open(file.fname);
			get_id3(&Mp3, &Tag);
			if (MusicP){
				send_endFillByte(2048);
				f_lseek(&Mp3, MusicP);
				set_volume(0xFE);
				MuteTimer = 25;
			}
			print_mp3_info(PlayMusic, MusicCount, &Tag);
		} else {
			beep(3, BEEP_LOW);
			JobStat = JOB_MP3 + 9;
			break;
		}
		if (!SwFlag){
			JobStat++;
		}
		break;

	  case 1:
		sw = SwFlag & (~last_sw);
		if (sw & SW_POWER){
			mp3_close();
			beep(1, BEEP_HIGH);
			power_off();
		} else if (sw & SW_LEFT){
			mp3_close();
			beep(1, BEEP_HIGH);
			if (PlayMusic == 1){
				PlayMusic = MusicCount;
			} else {
				--PlayMusic;
			}
			search_sound_file(PlayMusic, &file);
			mp3_open(file.fname);
			get_id3(&Mp3, &Tag);
			print_mp3_info(PlayMusic, MusicCount, &Tag);
		} else if (sw & SW_RIGHT){
			mp3_close();
			beep(1, BEEP_HIGH);
			// 次の曲
			MusicP = 0;
			if (++PlayMusic > MusicCount){
				PlayMusic = 1;
			}
			search_sound_file(PlayMusic, &file);
			mp3_open(file.fname);
			get_id3(&Mp3, &Tag);
			print_mp3_info(PlayMusic, MusicCount, &Tag);
		} else if (sw & SW_MODE){
			mp3_close();
			beep(1, BEEP_HIGH);
			JobStat = JOB_JPEG;
			return;
		} else if (sw & SW_A){
		} else if (sw & SW_B){
		}
		if (!VolumeTimer){
			if (SwFlag & SW_UP){
				if (Volume >= 4){
					Volume -= 4;
					
				}
				VolumeTimer = 25;
			} else if (SwFlag & SW_DOWN){
				if (Volume <= 0xA0){
					Volume += 4;
				}
				VolumeTimer = 25;
			}
		}
		break;

	  case 9:
		break;

	  default:
		JobStat = JOB_MP3;
		break;
	}

	print_decode_info();

	if (VS1053stat == VS1053_STOP){
		if (!pause_flag){
			// 次の曲
			MusicP = 0;
			if (++PlayMusic > MusicCount){
				PlayMusic = 1;
			}

			// スリープタイマー
			//if (SleepFlag && !SleepTimer){
			//	shutdown();	// 電源OFF
			//}

			search_sound_file(PlayMusic, &file);
			mp3_open(file.fname);	
			get_id3(&Mp3, &Tag);
			print_mp3_info(PlayMusic, MusicCount, &Tag);
			ret = -1;	// 再生終了
		} else {
			ret = 1;
		}
		prog = 0;
	} else {
		long	l;
		l = Mp3.fptr;
		l *= BAR_GRAPH_MAX;
		l += Mp3.fsize / 2;
		l /= Mp3.fsize;
		prog = (ushort)l;
	}
	if (!pause_flag && (prog != last_prog)){
		print_bar_graph(prog);
	}

	last_sw = SwFlag;
	last_prog = prog;

	return ret;
}


//*******************************************************
//	MP3 バーグラフ表示
//	leve は 0～BAR_GRAPH_MAX
//*******************************************************
void print_bar_graph(short level){

	if (level > BAR_GRAPH_MAX)	level = BAR_GRAPH_MAX;
	if (level < 0)	level = 0;

	if (level > 0){
		fill_rect(LCD_WIDTH - BAR_GRAPH_MAX - 1, LCD_HEIGHT - 5, LCD_WIDTH - BAR_GRAPH_MAX - 2 + level, LCD_HEIGHT - 2, COLOR_YELLOW);
	}
	if (level < BAR_GRAPH_MAX){
		fill_rect(LCD_WIDTH - BAR_GRAPH_MAX - 1 + level, LCD_HEIGHT - 5, LCD_WIDTH - 2, LCD_HEIGHT - 2, COLOR_BLACK);
	}
}


//********************************************************************
//	デコード情報表示
//********************************************************************
void print_decode_info(void){
	ushort		h0, h1;
	ulong		bps = 0;
	uchar		id;
	//ushort		old_bg;

	if (DecodeTimer)	return;
	DecodeTimer = 100;	// １秒ごとに表示
	
	h0 = MP3_command_read(SCI_HDAT0);
	h1 = MP3_command_read(SCI_HDAT1);

	//old_bg = BGcolor;
	//BGcolor = RGB(0, 0, 8);

	locate(8, 24);

	if (h1 == 0x574D){
		print("WMA      ");
		bps = h0;
	} else if (h1 == 0x4D54){
		print("MIDI     ");
		bps = h0;
	} else if (h1 == 0x4F67){
		print("Ogg      ");
		bps = h0;
	} else if (h1 == 0x4154){
		print("AAC ADTS ");
		bps = h0;
	} else if (h1 == 0x7665){
		print("WAVE     ");
		bps = h0;
	} else if (h1 == 0x664C){
		print("FLAC     ");
		bps = h0;
	} else if (h1 != 0){
		print("MP3  ");

		id = (h1 >> 3) & 3;

		if (id == 3){
			switch((h0 >> 10) & 3){
			  case 2:	print("32kHz/");	break;
			  case 1:	print("48kHz/");	break;
			  case 0:	print("44kHz/");	break;
			}
		} else if (id == 2){
			switch((h0 >> 10) & 3){
			  case 2:	print("16kHz/");	break;
			  case 1:	print("24kHz/");	break;
			  case 0:	print("22kHz/");	break;
			}
		} else {
			switch((h0 >> 10) & 3){
			  case 2:	print(" 8kHz/");	break;
			  case 1:	print("12kHz/");	break;
			  case 0:	print("11kHz/");	break;
			}
		}

		if (id == 3){
			switch(h0 >> 12){
			  case 14:	print("320kbps");	break;
			  case 13:	print("256kbps");	break;
			  case 12:	print("224kbps");	break;
			  case 11:	print("192kbps");	break;
			  case 10:	print("160kbps");	break;
			  case 9:	print("128kbps");	break;
			  case 8:	print("112kbps");	break;
			  case 7:	print("96kbps ");	break;
			  case 6:	print("80kbps ");	break;
			  case 5:	print("64kbps ");	break;
			  case 4:	print("56kbps ");	break;
			  case 3:	print("48kbps ");	break;
			  case 2:	print("40kbps ");	break;
			  case 1:	print("32kbps ");	break;
			}
		} else {
			switch(h0 >> 12){
			  case 14:	print("160kbps");	break;
			  case 13:	print("144kbps");	break;
			  case 12:	print("128kbps");	break;
			  case 11:	print("112kbps");	break;
			  case 10:	print("96kbps ");	break;
			  case 9:	print("80kbps ");	break;
			  case 8:	print("64kbps ");	break;
			  case 7:	print("56kbps ");	break;
			  case 6:	print("48kbps ");	break;
			  case 5:	print("40kbps ");	break;
			  case 4:	print("32kbps ");	break;
			  case 3:	print("24kbps ");	break;
			  case 2:	print("16kbps ");	break;
			  case 1:	print("8kbps  ");	break;
			}
		}
	}

	if (bps != 0){
		char	buff[12];
		bps *= 8;
		bps += 50;
		itoa(bps, buff, 10);
		if (bps >= 100000){
			buff[4] = buff[3];
			buff[5] = 0;
			buff[3] = '.';
		} else {
			buff[3] = buff[2];
			buff[4] = 0;
			buff[2] = '.';
			putch(' ');
		}
		print(buff);
		print("kbps");
	}

	//BGcolor = old_bg;
}


//********************************************************************
//	MP3 ID3タグ、曲数などの表示
//********************************************************************
void print_mp3_info(ushort n, ushort count, struct ID3_TAG *tag){
	char	msg[8];
	char	ft, fa, lt, la;

	// 文字の長さでフォントやレイアウトを選ぶ
	ft = 16;
	lt = check_line(tag->title, LCD_WIDTH - 8, ft);

	if (lt > 1){
		char	l;
		if ((l = check_line(tag->title, LCD_WIDTH - 8, 14)) < lt){
			ft = 14;
			lt = l;
		} else if ((l = check_line(tag->title, LCD_WIDTH - 8, 12)) < lt){
			ft = 12;
			lt = l;
		}
	}

	fa = 16;
	la = check_line(tag->artist, LCD_WIDTH - 8, fa);
	if (la > 1){
		char	l;
		if ((l = check_line(tag->artist, LCD_WIDTH - 8, 14)) < la){
			fa = 14;
			la = l;
		} else if ((l = check_line(tag->artist, LCD_WIDTH - 8, 12)) < la){
			fa = 12;
			la = l;
		}
	}

	if ((46 + lt * (ft + 2) + 4 + la * (fa + 2)) >= 121){
		if (la <= 1){
			lt = (121 - 46 - 4 - fa) / (ft + 2);
			trim_line(tag->title, ft, lt);
		} else if (lt <= 1){
			la = (121 - 46 - 4 - ft) / (fa + 2);
			trim_line(tag->artist, fa, la);
		} else {
			fa = ft = 12;
			lt = 2;
			trim_line(tag->title, ft, lt);
			la = (121 - 46 - 4 - ft) / (fa + 2);
			trim_line(tag->artist, fa, la);
		}
	}

	if (!tag->title[0] && !tag->artist[0]){
		strcpy_P(tag->title, PSTR("No TAG"));
	}
	
	// 表示エリアクリア
	fill_rect(0, 20, LCD_WIDTH - 1, LCD_HEIGHT - 7, BGcolor);

	word_wrap(tag->title, (LCD_WIDTH-8)/(ft/2));
	print_sjis(8, 46, ft, tag->title);
	
	word_wrap(tag->artist, (LCD_WIDTH-8)/(fa/2));
	print_sjis(8, lt * (ft + LINE_SPACE) + 46 + 4, fa, tag->artist);

	itoa(n, msg, 10);
	locate(LCD_WIDTH - 60, 34);
	print(msg);
	putch('/');
	itoa(count, msg, 10);
	print(msg);

	// 0.1秒後にデコード情報表示
	DecodeTimer = 10;
}


//*******************************************************
//	指定行数内に収まるように Shift_JIS文字列をカット
//*******************************************************
void trim_line(char *str, char font, char line){
	uchar	c;
	uchar	*p;
	char	l = 0;
	int		width = 0;
	char	w;

	while((c = (uchar)*str)){
		p = (uchar *)str;
		if (is_sjis1(c)){	// Shift_JIS
			w = font;
			str += 2;
		} else {
			w = font / 2;
			str++;
		}
		width += w;
		if (width > LCD_WIDTH - 8){
			if (++l >= line){
				*p = 0;
				break;
			}
			width = w + 0;
		}
	}
}


//********************************************************************
//	文字列の長さに応じてスペースで改行
//********************************************************************
void word_wrap(char str[], short width){
	uchar	i;

	if ((strlen(str) > width)&&
		(strlen(str) < (width * 2 ))){
		char	*p;

		p = &str[width];
		for (i = 1; i < (width * 2 - strlen(str)); i++){
			--p;
			if (*p == ' '){
				*p = LF;
				break;
			} else if (*p == '('){	// Shift_JISコードに 0x2F以下は含まない
				char	*s, *d;
				d = s = &str[strlen(str)];
				d++;
				while(s >= p){
					*d-- = *s--;
				}
				*p = LF;
				break;
			}
		}
	}
}


//********************************************************************
//	文字列の表示に何行必要か調べる
//********************************************************************
short check_line(char *sjis, short width, char font){
	short		line = 0;
	short		x = 0;
	uchar		c;

	while((c = *sjis++)){
		if (c == LF){
			x = 0;
		} else if (is_sjis1(c)){
			if (x == 0)	line++;
			sjis++;
			x += font;
			if (x > width){
				line++;
				x = font;
			}
		} else {
			if (x == 0)	line++;
			x += font / 2;
			if (x > width){
				line++;
				x = font / 2;
			}
		}
	}

	return line;
}


//********************************************************************
//	MP3再生開始
//********************************************************************
FRESULT	mp3_open(char *fname){
	FRESULT	rc;

	// ファイルオープン
	f_close(&Mp3);
	rc = f_open(&Mp3, fname, FA_READ);
	if (rc != FR_OK)	return rc;

	VS1053stat = VS1053_PLAY | VS1053_PLAY_FILE;

	BusyTimer = 100;

	return rc;
}


//********************************************************************
//	MP3 データ転送終了
//********************************************************************
void mp3_close(){

	f_close(&Mp3);

	if (VS1053stat == VS1053_STOP)	return;

	vs1053_stop();
	BusyTimer = 0;
}


//********************************************************************
//	サウンドファイルを探す
//********************************************************************
ushort search_sound_file(ushort num, FILINFO *file){
	DIR	dir;
	short	n = 0;
	char	path;

	path = 0;
	if (f_opendir(&dir, &path) != FR_OK)	return 0;

	while((f_readdir(&dir, file) == FR_OK)&&
		  (file->fname[0] != 0)){
		char	*p;
		if (file->fname[0] == '.')	continue;
		if (file->fattrib & (AM_DIR | AM_VOL))	continue;
		p = strchr(file->fname, '.');
		if ((p != NULL)&&
			((strcmp_P(p, PSTR(".MP3")) == 0)||
			 (strcmp_P(p, PSTR(".OGG")) == 0)||
			 (strcmp_P(p, PSTR(".WMA")) == 0)||
			 (strcmp_P(p, PSTR(".AAC")) == 0)||
			 (strcmp_P(p, PSTR(".M4A")) == 0)||
			 //(strcmp_p(p, PSTR(".WAV")) == 0)||
			 //(strcmp_P(p, PSTR(".FLA")) == 0)||
			 (strcmp_P(p, PSTR(".MID")) == 0))){
			n++;
			if (--num == 0)	break;
		}
	}
	
	return n;
}


