//*******************************************************
//
//	MP3 Player in SEGA Visual Memory
//
//	VS1053B処理
//
//	ATmega1284p
//		efuse:0xFD	BOD 2.7V
//		hfuse:0xD1	JTAG禁止, EEPROM消去保護
//		lfuse:0xE2	内蔵CR発振 8MHz
//
//	Compiler:	AVR-GCC		4.3.5
//				AVR-LIBC	1.6.8
//				binutils	2.20
//
//	2012.6.3	KAWAKAMI Yukio
//*******************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>
#include	<avr/eeprom.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include	"main.h"
#include	"lcd.h"
#include	"ff.h"
#include	"rom.h"
#include	"vs1053.h"
#include	"resource.h"


// テスト用連続正弦波出力データ		Beepとして使う
PROGMEM uchar BeepData[] = { 0x53, 0xEF, 0x6E, 0xF0, 0, 0, 0, 0 };	// 1500Hz
PROGMEM uchar ErrorBeep[] = { 0x53, 0xEF, 0x6E, 0xA4, 0, 0, 0, 0 };	// 500Hz
PROGMEM uchar BeepStop[] = { 0x45, 0x78, 0x69, 0x74, 0, 0, 0, 0 };	// 止める

volatile uchar	Volume;			// 音量
volatile uchar	VolumeReg;		// 現在の実際の音量設定値
volatile uchar	BeepTimer;		// Beep音の長さ
volatile uchar	BusyTimer;
volatile uchar	MuteTimer;
//volatile uchar	DecodeTimer;
//volatile ushort VolumeTimer;

ushort SaveHardVolume;
char PatchFlag = 0;


uchar	VS1053stat = 0;
FIL		Mp3;
ulong	Mp3Adrs;
ulong	Mp3Remain;
uchar	EndFillByte;
ushort	EndFillByteCount;

extern ushort	JobStat;

extern ulong	MusicP;		// 再生中の曲のポインタ

//********************************************************
//	VS1053 逐次処理
//********************************************************
void job_vs1053(void){

	if (!MuteTimer){
		set_volume(Volume);
	}

	switch(VS1053stat & 0x0F){
		uchar	*p, buff[32];
		UINT	n;
		FRESULT	res;

	  case VS1053_PLAY_FILE:
		// MP3ファイル再生
		while(mp3_check()){
			BusyTimer = 100;
			MusicP = Mp3.fptr;
			res = f_read(&Mp3, buff, sizeof(buff), &n);
			if (res != FR_OK){
				//char	msg[8];
				//locate(0, 100);
				//print("f_read error : ");
				//itoa(res, msg, 10);
				//print(msg);
				//wait_timer(200);
				n = 0;
			}
			if (n == 0){
				VS1053stat = VS1053_ENDING;
				break;
			}

			p = buff;
			XDCS_ON;
			while(n > 0){
				SPI_out(*p++);
				--n;
			}
			XDCS_OFF;
		}
		break;

	  case VS1053_PLAY_ROM:
		// シリアルフラッシュROM上のデータ再生
		while(mp3_check()){
			BusyTimer = 100;
			ROM_read(Mp3Adrs, buff, sizeof(buff));
			Mp3Adrs += sizeof(buff);
			if (Mp3Remain >= sizeof(buff)){
				Mp3Remain -= sizeof(buff);
				n = sizeof(buff);
			} else {
				n = Mp3Remain;
				Mp3Remain = 0;
			}

			p = buff;
			XDCS_ON;
			while(n > 0){
				SPI_out(*p++);
				--n;
			}
			XDCS_OFF;
			if (Mp3Remain == 0)	break;
		}
		if (Mp3Remain == 0)	VS1053stat = VS1053_ENDING;
		break;
	}

	switch(VS1053stat & 0xF0){
	  case VS1053_ENDING:
		switch(VS1053stat & 0x0F){
		  case 0:
			MP3_command(SCI_WRAMADDR, 0x1e06);
			EndFillByte = MP3_command_read(SCI_WRAM);
			EndFillByteCount = 0;
			VS1053stat = VS1053_ENDING | 8;	// ↓続けて実行
		  case 8:
			while(mp3_check()){
				int	i;
				BusyTimer = 100;
				XDCS_ON;
				for (i = 0; i < 32; i++){
					SPI_out(EndFillByte);
				}
				XDCS_OFF;
				EndFillByteCount += 32;
				if (EndFillByteCount >= 2052)	break;
			}
			if (EndFillByteCount >= 2052){
				MP3_command(SCI_MODE, MP3_command_read(SCI_MODE) | SM_CANCEL);
				MP3_command(SCI_WRAMADDR, 0x1e06);
				EndFillByte = MP3_command_read(SCI_WRAM);
				EndFillByteCount = 0;
				VS1053stat++;
			}
			break;

		  case 9:
			if (mp3_check()){
				int	i;
				BusyTimer = 100;
				XDCS_ON;
				for (i = 0; i < 32; i++){
					SPI_out(EndFillByte);
				}
				XDCS_OFF;
				EndFillByteCount += 32;
			}
			if (EndFillByteCount > 2048){
				MP3_init();
				VS1053stat = VS1053_STOP;
			} else if (EndFillByteCount >= 32){
				if (!(MP3_command_read(SCI_MODE) & SM_CANCEL)){
					VS1053stat = VS1053_STOP;
				}
			}
			break;
		}
		break;

	  case VS1053_STOPPING:
		MP3_command(SCI_MODE, MP3_command_read(SCI_MODE) | SM_CANCEL);
		EndFillByteCount = 0;
		VS1053stat = VS1053_STOPPING2 & (VS1053stat & 0x0F);
		break;

	  case VS1053_STOPPING2:
		EndFillByteCount += 32;
		if (!(MP3_command_read(SCI_MODE) & SM_CANCEL)){
			MP3_command(SCI_WRAMADDR, 0x1e06);
			EndFillByte = MP3_command_read(SCI_WRAM);
			EndFillByteCount = 0;
			VS1053stat = VS1053_STOPPING3;
		} else if (EndFillByteCount >= 2048){
			MP3_init();
			VS1053stat = VS1053_STOP;
		}
		break;

	  case VS1053_STOPPING3:
		while (mp3_check()){
			int	i;
			BusyTimer = 100;
			XDCS_ON;
			for (i = 0; i < 32; i++){
				SPI_out(EndFillByte);
			}
			XDCS_OFF;
			EndFillByteCount += 32;
			if (EndFillByteCount >= 2052){
				VS1053stat = VS1053_STOP;
				break;
			}
		}
		break;

	  case VS1053_JUMP:
		if (!(MP3_command_read(SCI_STATUS) & SS_DO_NOT_JUMP)){
			MP3_command(SCI_WRAMADDR, 0x1e06);
			EndFillByte = MP3_command_read(SCI_WRAM);
			EndFillByteCount = 0;
			VS1053stat = VS1053_STOPPING3;
		}
		break;

	  default:
		break;
	}
}


//********************************************************
//	MP3再生中断する
//********************************************************
void vs1053_stop(void){
	
	if (VS1053stat != VS1053_STOP){
		VS1053stat = VS1053_STOPPING | (VS1053stat & 0x0F);
		while (VS1053stat != VS1053_STOP){
			job_vs1053();
		}
	}
}


//********************************************************
//	リソースの MP3ファイルを再生する BEEP
//********************************************************
void beep(uchar n, char err){
	ulong	adrs, remain;

	if (VS1053stat != VS1053_STOP){
		vs1053_stop();
		beep_wait(45);	// 残りを再生していることがある？
	}

	beep_wait(5);

	set_volume(Volume);

	if (!err){
		resource_adrs_size(MP3_BEEP_HIGH, &adrs, &remain);
	} else {
		resource_adrs_size(MP3_BEEP_LOW, &adrs, &remain);
	}

	while (n > 0){
		BeepTimer = 13;
		VS1053stat = VS1053_PLAY | VS1053_PLAY_ROM;
		Mp3Adrs = adrs;
		Mp3Remain = remain;
		while(VS1053stat != VS1053_STOP){
			job_vs1053();
		}
		while(BeepTimer){
			WDR;
			SLEEP;
		}
		--n;
	}
}


//********************************************************
//	実際にレジスタ操作して音量設定
//********************************************************
void set_volume(uchar vol){

	if (vol != VolumeReg){
		MP3_command(SCI_VOL, ((ushort)vol << 8)|((ushort)vol));	// 音量設定
		VolumeReg = vol;
	}
}


//********************************************************
//	VS1053 ハードウェアボリューム設定
//********************************************************
void set_hd_volume(uchar vol){
	ushort			reg;
	static uchar	hv = 0;

	if (vol == hv)	return;

	if (vol == 0){
		reg = 0;
	} else {
		ushort	low;
		ushort	high;

		if (vol >= 0x10)	vol -= 0x10;	// MP3とFMの音量差

		if (vol > 168)	vol = 168;
		
		low = (vol-1) / 12 + 1;
		high = (11 - ((vol-1) % 12)) << 4;
		reg = high | low | (high << 8) | (low << 8);
	}

	MP3_command(SCI_WRAMADDR, 0xC045);
	MP3_command(SCI_WRAM, reg);
}


//****************************************
//	VS1053の sinテストを用いた BEEP
//	引数：	n	鳴動回数
//			err	!= 0 ならエラー音(低い音)
//	SLEEPを使うので割り込み許可で呼び出すこと
//	解除するには SOFTリセットが必要
//****************************************
void test_beep(uchar n, char err){
	uchar	vol;

	MP3_init();

#if BEEP_VOL_DIFF
	// ボリュームを下げる
	vol = Volume + BEEP_VOL_DIFF;
	if (vol < BEEP_VOL_DIFF){
		vol = Volume;	// 音量変更なし
	}
#else
	vol = Volume;	// 音量変更なし
#endif

	beep_wait(10);	// 100ms	アンプミュートからの復帰待ち

	MP3_wait();
	MP3_command(SCI_MODE, SM_SDINEW | SM_TESTS);	// テストモード
	if (err){
		MP3_sin((PGM_VOID_P)ErrorBeep);
	} else {
		MP3_sin((PGM_VOID_P)BeepData);
	}

	for (; n > 1; --n){
		// 時間待ち。この間が BEEPの音の長さになる
		MP3_command(SCI_VOL, ((ushort)vol << 8)|((ushort)vol));	// 音量設定
		beep_wait(10);	// 100ms
		MP3_command(SCI_VOL, 0xFEFE);	// 音量最小
		// BEEPの音間 約30ms
		beep_wait(3);
	}
	MP3_command(SCI_VOL, ((ushort)vol << 8)|((ushort)vol));	// 音量設定
	beep_wait(10);	// 100ms

	MP3_sin((PGM_VOID_P)BeepStop);

	//MP3_command(SCI_MODE, SM_SDINEW);	// テストモード解除

	MP3_init();
}


//****************************************
//	beep()内の時間待ち
//****************************************
void beep_wait(uchar n){

	BeepTimer = n;
	while(BeepTimer){
		WDR;
		SLEEP;
	}
}


//****************************************
//	VS1053b デコード中断
//****************************************
void MP3_stop(void){
}


//****************************************
//	VS1053 初期化
//	SM_RESET設定後はパッチを再設定する必要がある
//****************************************
void MP3_init(void){

	SPCR = (1<<SPE)|(1<<MSTR);
	SPSR = 0;	// ここではまだ SPIクロックは遅くしておく
	SPI_in();

	MP3_wait();

	MP3_command(SCI_MODE, SM_SDINEW | SM_RESET);			// Soft Reset
	MP3_wait();

	// XTALI 12.288MHz, CLKI 3X = 36.864MHz, MAX 12.288 x 3 x 1.5 = 55.3MHz
	MP3_command(SCI_CLOCKF, (12288000L-8000000L)/4000L + 0x6000 + 0x1000);

	// XTALI 12.288MHz, CLKI 2.5X = 30.72MHz, MAX 12.288 x 2.5 x 1.5 = 46MHz
	//MP3_command(SCI_CLOCKF, (12288000L-8000000L)/4000L + 0x4000 + 0x1000);

	MP3_command(SCI_VOL, ((ushort)Volume << 8)|((ushort)Volume));	// 音量設定

	// VS1053の CLKIを 36.8MHzにしたので 36.8/7=5.2MHzまでの SPIクロックが可能
	SPSR = (1<<SPI2X);	// 4MHz

	PatchFlag = 0;
}


//****************************************
//	VS1053 テスト用正弦波出力
//****************************************
void MP3_sin(PGM_VOID_P data){
	uchar	i;
	uchar	c;

	XCS_OFF;
	XDCS_ON;

	for (i = 0; i < 8; i++){
		SPI_out(0);
	}
	for (i = 0; i < 8; i++){
		c = (uchar)pgm_read_byte(data++);
		SPI_out(c);
	}
	// 余分にゼロを送る こうしないと STOP時、音が鳴りっぱなしになることがある
	for (i = 0; i < 8; i++){	// 個数は適当。減らしてもいいかもしれない
		SPI_out(0);
	}

	XDCS_OFF;
}


//****************************************
//	VS1053 コマンド終了待ち
//****************************************
void MP3_wait(void){

	InitWait();
	while(!CHECK_DREQ);
}


//****************************************
//	VS1053 コマンド出力
//****************************************
void MP3_command(uchar addr, ushort arg){

	MP3_wait();

	XDCS_OFF;
	XCS_ON;

	SPI_out(2);
	SPI_out(addr);
	SPI_out(arg >> 8);
	SPI_out(arg);

	XCS_OFF;
}


//****************************************
//	VS1053 コマンド入力
//****************************************
ushort MP3_command_read(uchar addr){
	ushort	r;

	while(!CHECK_DREQ);

	XCS_ON;

	SPI_out(3);
	SPI_out(addr);
	r = ((ushort)SPI_in()) << 8;
	r |= SPI_in();
	XCS_OFF;

	return r;
}


//****************************************
//	初期化時のクロックウエイト
//****************************************
void InitWait(void){
	uchar	i;

	for(i = 0; i < 255; i++){
		NOP;
		NOP;
	}
}


//********************************************************************
//	2052bytesの endFillByte送信
//********************************************************************
void send_endFillByte(ushort n){
	uchar	endFillByte;

	MP3_command(SCI_WRAMADDR, 0x1e06);
	endFillByte = MP3_command_read(SCI_WRAM);
	XDCS_ON;
	while(n > 0){
		while(!CHECK_DREQ){
			WDR;
		}
		SPI_out(endFillByte);
		--n;
	}
	XDCS_OFF;
}


//********************************************************************
//	ヘッドフォンジャックのチェック
//	戻り値：	0:なし	1:接続されている
//********************************************************************
char check_jack(void){

	if (PINA & (1<<PLUG)){
		return 0;
	} else {
		return 1;
	}
}


//********************************************************************
//	アンプのミュート
//********************************************************************
void amp_mute(uchar stat){

	if (stat){
		// 内蔵スピーカーアンプミュート
		PORTA &= ~(1<<AMP);
	} else {
		// 内蔵スピーカーアンプ ON
		PORTA |= (1<<AMP);
	}
}


//********************************************************************
//	ヘッドフォンジャックの状態に応じてスピーカーアンプミュート
//********************************************************************
void set_jack_mute(void){

	if (check_jack()){
		amp_mute(ON);
	} else {
		amp_mute(OFF);
	}
}


//*********************************************************
//	ADPCM録音モード ON
//	VS1053bの場合、こうすると LINE入力がヘッドフォンモニタできる
//	モード切り替えにソフトウェアリセットを実行する
//*********************************************************
void ADPCM_mode_on(void){

	// ハードウェアボリューム値保存
	MP3_command(SCI_WRAMADDR, 0xC045);
	SaveHardVolume = MP3_command_read(SCI_WRAM);

	MP3_command(SCI_AICTRL0, 32000);	// 32kHz
	MP3_command(SCI_AICTRL1, 1024);
	//MP3_command(SCI_AICTRL2, 1024);
	MP3_command(SCI_AICTRL2, 0);
	//MP3_command(SCI_AICTRL3, 4);
	MP3_command(SCI_AICTRL3, 0);
	MP3_command(SCI_MODE, SM_SDINEW | SM_ADPCM | SM_LINE1 | SM_RESET);

	PatchFlag = 0;

	MP3_wait();

	// ハードウェアボリューム設定
	set_hd_volume(Volume);
}


//*********************************************************
//	ADPCM録音モード OFF
//	モード切り替えにソフトウェアリセットを実行する
//*********************************************************
void ADPCM_mode_off(void){

	MP3_init();

	// ハードウェアボリュームを元に戻す
	MP3_command(SCI_WRAMADDR, 0xC045);
	MP3_command(SCI_WRAM, SaveHardVolume);

	beep_wait(5);
	set_volume(Volume);
}


