#ifndef	__VS1053_H__
#define	__VS1053_H__
//*******************************************************
//
//	MP3 Player in SEGA Visual Memory
//
//	VS1053B処理
//
//	ATmega1284p
//		efuse:0xFD	BOD 2.7V
//		hfuse:0xD1	JTAG禁止, EEPROM消去保護
//		lfuse:0xE2	内蔵CR発振 8MHz
//
//	Compiler:	AVR-GCC		4.3.5
//				AVR-LIBC	1.6.8
//				binutils	2.20
//
//	2012.6.3	KAWAKAMI Yukio
//*******************************************************

// VS1053 レジスタ・ビット
#define	SCI_MODE		0
#define	SM_RESET		(1<<2)
#define	SM_CANCEL		(1<<3)
#define	SM_TESTS		(1<<5)
#define	SM_SDINEW		(1<<11)
#define	SM_ADPCM		(1<<12)
#define	SM_LINE1		(1<<14)
#define	SM_CLK_RANGE	(1<<15)

#define	SCI_STATUS		1
#define	SS_APDOWN2		(1<<3)
#define	SS_APDOWN1		(1<<2)
#define	SS_DO_NOT_JUMP	(1<<15)

#define	SCI_BASS		2
#define	SCI_CLOCKF		3
#define	SCI_DECODE_TIME	4
#define	SCI_AUDATA		5
#define	SCI_WRAM		6
#define	SCI_WRAMADDR	7
#define	SCI_HDAT0		8
#define	SCI_HDAT1		9
#define	SCI_VOL			11
#define SCI_AICTRL0		12
#define SCI_AICTRL1		13
#define SCI_AICTRL2		14
#define SCI_AICTRL3		15

//#define	VS_GPIO_JACK	(1<<7)
//#define	VS_GPIO_AMP		(1<<5)

// VS1053 CS, DREQ, RESET
#define		XDCS_ON		PORTB &= ~(1<<XDCS)
#define		XDCS_OFF	PORTB |= (1<<XDCS)
#define		XCS_ON		PORTB &= ~(1<<XCS)
#define		XCS_OFF		PORTB |= (1<<XCS)
#define		CHECK_DREQ	(PINB & (1<<DREQ))
#define		XRES_ON		PORTA &= ~(1<<XRES)
#define		XRES_OFF	PORTA |= (1<<XRES)

#define	mp3_check()	(CHECK_DREQ)

#define	BEEP_HIGH	0
#define	BEEP_LOW	1

#define	VS1053_STOP			0
#define	VS1053_PLAY_FILE	1
#define	VS1053_PLAY_ROM		2
#define	VS1053_PLAY			0x10
#define	VS1053_STOPPING		0x20
#define	VS1053_STOPPING2	0x30
#define	VS1053_STOPPING3	0x40
#define	VS1053_JUMP			0x50
#define	VS1053_ENDING		0x60
#define	VS1053_RESET		0x90

void job_vs1053(void);

void vs1053_stop(void);

char check_jack(void);
void amp_mute(uchar stat);
void set_jack_mute(void);

void send_endFillByte(ushort n);
void set_volume(uchar vol);
void set_hd_volume(uchar vol);
void beep(uchar n, char err);
void test_beep(uchar n, char err);
void beep_wait(uchar n);
void MP3_stop(void);
void MP3_init(void);
void MP3_sin(PGM_VOID_P data);
void MP3_wait(void);
void MP3_command(uchar addr, ushort arg);
ushort MP3_command_read(uchar addr);
void InitWait(void);

void ADPCM_mode_on(void);
void ADPCM_mode_off(void);


#endif	// __VS1053_H__
